@@ -1,16 +1,29 @@
 <?php
 /**
- * Copyright (c) 2015 Vincent Petry <pvince81@owncloud.com>
- * This file is licensed under the Affero General Public License version 3 or
- * later.
- * See the COPYING-README file.
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Vincent Petry <pvince81@owncloud.com>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
  */
 
 namespace OCA\Files_trashbin\Tests\Storage;
 
-use OC\Files\Storage\Home;
 use OC\Files\Storage\Temporary;
-use OC\Files\Mount\MountPoint;
 use OC\Files\Filesystem;
 
 class Storage extends \Test\TestCase {
@@ -19,11 +32,6 @@ class Storage extends \Test\TestCase {
 	 */
 	private $user;
 
-	/**
-	 * @var \OC\Files\Storage\Storage
-	 **/
-	private $originalStorage;
-
 	/**
 	 * @var \OC\Files\View
 	 */
@@ -46,8 +54,6 @@ class Storage extends \Test\TestCase {
 		// this will setup the FS
 		$this->loginAsUser($this->user);
 
-		$this->originalStorage = \OC\Files\Filesystem::getStorage('/');
-
 		\OCA\Files_Trashbin\Storage::setupStorage();
 
 		$this->rootView = new \OC\Files\View('/');
@@ -60,7 +66,6 @@ class Storage extends \Test\TestCase {
 
 	protected function tearDown() {
 		\OC\Files\Filesystem::getLoader()->removeStorageWrapper('oc_trashbin');
-		\OC\Files\Filesystem::mount($this->originalStorage, array(), '/');
 		$this->logout();
 		\OC_User::deleteUser($this->user);
 		\OC_Hook::clear();
@@ -233,7 +238,6 @@ class Storage extends \Test\TestCase {
 	 */
 	public function testDeleteVersionsOfFileAsRecipient() {
 		\OCA\Files_Versions\Hooks::connectHooks();
-		\OCA\Files_Sharing\Helper::registerHooks();
 
 		$this->userView->mkdir('share');
 		// trigger a version (multiple would not work because of the expire logic)
@@ -281,7 +285,6 @@ class Storage extends \Test\TestCase {
 	 */
 	public function testDeleteVersionsOfFolderAsRecipient() {
 		\OCA\Files_Versions\Hooks::connectHooks();
-		\OCA\Files_Sharing\Helper::registerHooks();
 
 		$this->userView->mkdir('share');
 		$this->userView->mkdir('share/folder');
@@ -429,21 +432,28 @@ class Storage extends \Test\TestCase {
 		 */
 		$storage = $this->getMockBuilder('\OC\Files\Storage\Temporary')
 			->setConstructorArgs([[]])
-			->setMethods(['rename', 'unlink'])
+			->setMethods(['rename', 'unlink', 'moveFromStorage'])
 			->getMock();
 
+		$storage->expects($this->any())
+			->method('rename')
+			->will($this->returnValue(false));
+		$storage->expects($this->any())
+			->method('moveFromStorage')
+			->will($this->returnValue(false));
 		$storage->expects($this->any())
 			->method('unlink')
 			->will($this->returnValue(false));
 
 		$cache = $storage->getCache();
 
-		Filesystem::mount($storage, [], '/' . $this->user . '/files');
+		Filesystem::mount($storage, [], '/' . $this->user);
+		$storage->mkdir('files');
 		$this->userView->file_put_contents('test.txt', 'foo');
-		$this->assertTrue($storage->file_exists('test.txt'));
+		$this->assertTrue($storage->file_exists('files/test.txt'));
 		$this->assertFalse($this->userView->unlink('test.txt'));
-		$this->assertTrue($storage->file_exists('test.txt'));
-		$this->assertTrue($cache->inCache('test.txt'));
+		$this->assertTrue($storage->file_exists('files/test.txt'));
+		$this->assertTrue($cache->inCache('files/test.txt'));
 
 		// file should not be in the trashbin
 		$results = $this->rootView->getDirectoryContent($this->user . '/files_trashbin/files/');
@@ -468,18 +478,62 @@ class Storage extends \Test\TestCase {
 
 		$cache = $storage->getCache();
 
-		Filesystem::mount($storage, [], '/' . $this->user . '/files');
+		Filesystem::mount($storage, [], '/' . $this->user);
+		$storage->mkdir('files');
 		$this->userView->mkdir('folder');
 		$this->userView->file_put_contents('folder/test.txt', 'foo');
-		$this->assertTrue($storage->file_exists('folder/test.txt'));
-		$this->assertFalse($this->userView->rmdir('folder'));
-		$this->assertTrue($storage->file_exists('folder'));
-		$this->assertTrue($storage->file_exists('folder/test.txt'));
-		$this->assertTrue($cache->inCache('folder'));
-		$this->assertTrue($cache->inCache('folder/test.txt'));
+		$this->assertTrue($storage->file_exists('files/folder/test.txt'));
+		$this->assertFalse($this->userView->rmdir('files/folder'));
+		$this->assertTrue($storage->file_exists('files/folder'));
+		$this->assertTrue($storage->file_exists('files/folder/test.txt'));
+		$this->assertTrue($cache->inCache('files/folder'));
+		$this->assertTrue($cache->inCache('files/folder/test.txt'));
 
 		// file should not be in the trashbin
 		$results = $this->rootView->getDirectoryContent($this->user . '/files_trashbin/files/');
 		$this->assertEquals(0, count($results));
 	}
+
+	/**
+	 * @dataProvider dataTestShouldMoveToTrash
+	 */
+	public function testShouldMoveToTrash($mountPoint, $path, $userExists, $expected) {
+		$tmpStorage = $this->getMockBuilder('\OC\Files\Storage\Temporary')
+			->disableOriginalConstructor()->getMock();
+		$userManager = $this->getMockBuilder('OCP\IUserManager')
+			->disableOriginalConstructor()->getMock();
+		$userManager->expects($this->any())
+			->method('userExists')->willReturn($userExists);
+		$storage = new \OCA\Files_Trashbin\Storage(
+			['mountPoint' => $mountPoint, 'storage' => $tmpStorage],
+			$userManager
+		);
+
+		$this->assertSame($expected,
+			$this->invokePrivate($storage, 'shouldMoveToTrash', [$path])
+		);
+
+	}
+
+	public function dataTestShouldMoveToTrash() {
+		return [
+			['/schiesbn/', '/files/test.txt', true, true],
+			['/schiesbn/', '/files/test.txt', false, false],
+			['/schiesbn/', '/test.txt', true, false],
+			['/schiesbn/', '/test.txt', false, false],
+		];
+	}
+
+	/**
+	 * Test that deleting a file doesn't error when nobody is logged in
+	 */
+	public function testSingleStorageDeleteFileLoggedOut() {
+		$this->logout();
+
+		if (!$this->userView->file_exists('test.txt')) {
+			$this->markTestSkipped('Skipping since the current home storage backend requires the user to logged in');
+		} else {
+			$this->userView->unlink('test.txt');
+		}
+	}
 }
