@@ -1,10 +1,35 @@
 <?php
 /**
- * Copyright (c) 2012 Frank Karlitschek <frank@owncloud.org>
- *               2013 Bjoern Schiessle <schiessle@owncloud.com>
- * This file is licensed under the Affero General Public License version 3 or
- * later.
- * See the COPYING-README file.
+ * @author Bart Visscher <bartv@thisnet.nl>
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Felix Moeller <mail@felixmoeller.de>
+ * @author Florin Peter <github@florin-peter.de>
+ * @author Georg Ehrke <georg@owncloud.com>
+ * @author Joas Schilling <nickvergessen@owncloud.com>
+ * @author Jörn Friedrich Dreyer <jfd@butonic.de>
+ * @author Lukas Reschke <lukas@owncloud.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
+ * @author Scrutinizer Auto-Fixer <auto-fixer@scrutinizer-ci.com>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Vincent Petry <pvince81@owncloud.com>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
  */
 
 /**
@@ -15,6 +40,8 @@
 
 namespace OCA\Files_Versions;
 
+use OCA\Files_Versions\Command\Expire;
+
 class Storage {
 
 	const DEFAULTENABLED=true;
@@ -125,27 +152,13 @@ class Storage {
 			// create all parent folders
 			self::createMissingDirectories($filename, $users_view);
 
-			$versionsSize = self::getVersionsSize($uid);
-
-			// assumption: we need filesize($filename) for the new version +
-			// some more free space for the modified file which might be
-			// 1.5 times as large as the current version -> 2.5
-			$neededSpace = $files_view->filesize($filename) * 2.5;
-
-			self::expire($filename, $versionsSize, $neededSpace);
-
-			// disable proxy to prevent multiple fopen calls
-			$proxyStatus = \OC_FileProxy::$enabled;
-			\OC_FileProxy::$enabled = false;
+			self::scheduleExpire($uid, $filename);
 
 			// store a new version of a file
 			$mtime = $users_view->filemtime('files/' . $filename);
 			$users_view->copy('files/' . $filename, 'files_versions/' . $filename . '.v' . $mtime);
 			// call getFileInfo to enforce a file cache entry for the new version
 			$users_view->getFileInfo('files_versions/' . $filename . '.v' . $mtime);
-
-			// reset proxy state
-			\OC_FileProxy::$enabled = $proxyStatus;
 		}
 	}
 
@@ -257,7 +270,7 @@ class Storage {
 
 		// if we moved versions directly for a file, schedule expiration check for that file
 		if (!$rootView->is_dir('/' . $targetOwner . '/files/' . $targetPath)) {
-			self::expire($targetPath);
+			self::scheduleExpire($targetOwner, $targetPath);
 		}
 
 	}
@@ -274,30 +287,42 @@ class Storage {
 			// add expected leading slash
 			$file = '/' . ltrim($file, '/');
 			list($uid, $filename) = self::getUidAndFilename($file);
+			if ($uid === null || trim($filename, '/') === '') {
+				return false;
+			}
 			$users_view = new \OC\Files\View('/'.$uid);
 			$files_view = new \OC\Files\View('/'.\OCP\User::getUser().'/files');
+
+			if (!$files_view->isUpdatable($filename)) {
+				return false;
+			}
+
 			$versionCreated = false;
 
 			//first create a new version
 			$version = 'files_versions'.$filename.'.v'.$users_view->filemtime('files'.$filename);
 			if ( !$users_view->file_exists($version)) {
 
-				// disable proxy to prevent multiple fopen calls
-				$proxyStatus = \OC_FileProxy::$enabled;
-				\OC_FileProxy::$enabled = false;
-
 				$users_view->copy('files'.$filename, 'files_versions'.$filename.'.v'.$users_view->filemtime('files'.$filename));
 
-				// reset proxy state
-				\OC_FileProxy::$enabled = $proxyStatus;
-
 				$versionCreated = true;
 			}
 
+			$fileToRestore =  'files_versions' . $filename . '.v' . $revision;
+
+			$oldFileInfo = $users_view->getFileInfo($fileToRestore);
+			$newFileInfo = $files_view->getFileInfo($filename);
+			$cache = $newFileInfo->getStorage()->getCache();
+			$cache->update(
+				$newFileInfo->getId(), [
+					'size' => $oldFileInfo->getSize()
+				]
+			);
+
 			// rollback
 			if (self::copyFileContents($users_view, 'files_versions' . $filename . '.v' . $revision, 'files' . $filename)) {
 				$files_view->touch($file, $revision);
-				Storage::expire($file);
+				Storage::scheduleExpire($uid, $file);
 				return true;
 			} else if ($versionCreated) {
 				self::deleteVersion($users_view, $version);
@@ -320,20 +345,7 @@ class Storage {
 		list($storage1, $internalPath1) = $view->resolvePath($path1);
 		list($storage2, $internalPath2) = $view->resolvePath($path2);
 
-		if ($storage1 === $storage2) {
-			return $storage1->rename($internalPath1, $internalPath2);
-		}
-		$source = $storage1->fopen($internalPath1, 'r');
-		$target = $storage2->fopen($internalPath2, 'w');
-		// FIXME: might need to use part file to avoid concurrent writes
-		// (this would be an issue anyway when renaming/restoring cross-storage)
-		list(, $result) = \OC_Helper::streamCopy($source, $target);
-		fclose($source);
-		fclose($target);
-
-		if ($result !== false) {
-			$storage1->unlink($internalPath1);
-		}
+		$result = $storage2->moveFromStorage($storage1, $internalPath1, $internalPath2);
 
 		return ($result !== false);
 	}
@@ -347,6 +359,9 @@ class Storage {
 	 */
 	public static function getVersions($uid, $filename, $userFullPath = '') {
 		$versions = array();
+		if (empty($filename)) {
+			return $versions;
+		}
 		// fetch for old versions
 		$view = new \OC\Files\View('/' . $uid . '/');
 
@@ -530,12 +545,30 @@ class Storage {
 	}
 
 	/**
-	 * Erase a file's versions which exceed the set quota
+	 * Schedule versions expiration for the given file
+	 *
+	 * @param string $uid owner of the file
+	 * @param string $fileName file/folder for which to schedule expiration
 	 */
-	private static function expire($filename, $versionsSize = null, $offset = 0) {
+	private static function scheduleExpire($uid, $fileName) {
+		$command = new Expire($uid, $fileName);
+		\OC::$server->getCommandBus()->push($command);
+	}
+
+	/**
+	 * Expire versions which exceed the quota
+	 *
+	 * @param $filename
+	 * @return bool|int|null
+	 */
+	public static function expire($filename) {
 		$config = \OC::$server->getConfig();
 		if($config->getSystemValue('files_versions', Storage::DEFAULTENABLED)=='true') {
 			list($uid, $filename) = self::getUidAndFilename($filename);
+			if (empty($filename)) {
+				// file maybe renamed or deleted
+				return false;
+			}
 			$versionsFileview = new \OC\Files\View('/'.$uid.'/files_versions');
 
 			// get available disk space for user
@@ -552,9 +585,7 @@ class Storage {
 			}
 
 			// make sure that we have the current size of the version history
-			if ( $versionsSize === null ) {
-				$versionsSize = self::getVersionsSize($uid);
-			}
+			$versionsSize = self::getVersionsSize($uid);
 
 			// calculate available space for version history
 			// subtract size of files and current versions size from quota
@@ -563,12 +594,12 @@ class Storage {
 				$rootInfo = $files_view->getFileInfo('/', false);
 				$free = $quota-$rootInfo['size']; // remaining free space for user
 				if ( $free > 0 ) {
-					$availableSpace = ($free * self::DEFAULTMAXSIZE / 100) - ($versionsSize + $offset); // how much space can be used for versions
+					$availableSpace = ($free * self::DEFAULTMAXSIZE / 100) - $versionsSize; // how much space can be used for versions
 				} else {
-					$availableSpace = $free - $versionsSize - $offset;
+					$availableSpace = $free - $versionsSize;
 				}
 			} else {
-				$availableSpace = $quota - $offset;
+				$availableSpace = $quota;
 			}
 
 			$allVersions = Storage::getVersions($uid, $filename);
