@@ -1,23 +1,38 @@
 <?php
 /**
-* ownCloud
-*
-* @author Michael Gapczynski
-* @copyright 2012 Michael Gapczynski mtgap@owncloud.com
-*
-* This library is free software; you can redistribute it and/or
-* modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
-* License as published by the Free Software Foundation; either
-* version 3 of the License, or any later version.
-*
-* This library is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU AFFERO GENERAL PUBLIC LICENSE for more details.
-*
-* You should have received a copy of the GNU Affero General Public
-* License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * @author Arthur Schiwon <blizzz@owncloud.com>
+ * @author Bart Visscher <bartv@thisnet.nl>
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Craig Morrissey <craig@owncloud.com>
+ * @author dampfklon <me@dampfklon.de>
+ * @author Joas Schilling <nickvergessen@owncloud.com>
+ * @author Jörn Friedrich Dreyer <jfd@butonic.de>
+ * @author Lukas Reschke <lukas@owncloud.com>
+ * @author Michael Gapczynski <GapczynskiM@gmail.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Ramiro Aparicio <rapariciog@gmail.com>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Roeland Jago Douma <roeland@famdouma.nl>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Thomas Tanghus <thomas@tanghus.net>
+ * @author Vincent Petry <pvince81@owncloud.com>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
+ */
 
 OC_JSON::checkLoggedIn();
 OCP\JSON::callCheck();
@@ -31,10 +46,11 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 				try {
 					$shareType = (int)$_POST['shareType'];
 					$shareWith = $_POST['shareWith'];
-					$itemSourceName = isset($_POST['itemSourceName']) ? $_POST['itemSourceName'] : null;
+					$itemSourceName = isset($_POST['itemSourceName']) ? (string)$_POST['itemSourceName'] : null;
 					if ($shareType === OCP\Share::SHARE_TYPE_LINK && $shareWith == '') {
 						$shareWith = null;
 					}
+ 					$itemSourceName=(isset($_POST['itemSourceName'])) ? (string)$_POST['itemSourceName']:'';
 
 					$token = OCP\Share::shareItem(
 						$_POST['itemType'],
@@ -43,14 +59,18 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 						$shareWith,
 						$_POST['permissions'],
 						$itemSourceName,
-						(!empty($_POST['expirationDate']) ? new \DateTime($_POST['expirationDate']) : null)
+						(!empty($_POST['expirationDate']) ? new \DateTime((string)$_POST['expirationDate']) : null)
 					);
 
+					$response = [];
 					if (is_string($token)) {
-						OC_JSON::success(array('data' => array('token' => $token)));
-					} else {
-						OC_JSON::success();
+						$item = OCP\Share::getShareByToken($token, false);
+						$response['data'] = array('token' => $token,
+							'expiration' => $item['expiration']);
 					}
+					OC_JSON::success($response);
+				} catch (\OC\HintException $exception) {
+					OC_JSON::error(array('data' => array('message' => $exception->getHint())));
 				} catch (Exception $exception) {
 					OC_JSON::error(array('data' => array('message' => $exception->getMessage())));
 				}
@@ -61,20 +81,20 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 				if ((int)$_POST['shareType'] === OCP\Share::SHARE_TYPE_LINK && $_POST['shareWith'] == '') {
 					$shareWith = null;
 				} else {
-					$shareWith = $_POST['shareWith'];
+					$shareWith = (string)$_POST['shareWith'];
 				}
-				$return = OCP\Share::unshare($_POST['itemType'], $_POST['itemSource'], $_POST['shareType'], $shareWith);
+				$return = OCP\Share::unshare((string)$_POST['itemType'],(string) $_POST['itemSource'], (int)$_POST['shareType'], $shareWith);
 				($return) ? OC_JSON::success() : OC_JSON::error();
 			}
 			break;
 		case 'setPermissions':
 			if (isset($_POST['shareType']) && isset($_POST['shareWith']) && isset($_POST['permissions'])) {
 				$return = OCP\Share::setPermissions(
-					$_POST['itemType'],
-					$_POST['itemSource'],
-					$_POST['shareType'],
-					$_POST['shareWith'],
-					$_POST['permissions']
+					(string)$_POST['itemType'],
+					(string)$_POST['itemSource'],
+					(int)$_POST['shareType'],
+					(string)$_POST['shareWith'],
+					(int)$_POST['permissions']
 				);
 				($return) ? OC_JSON::success() : OC_JSON::error();
 			}
@@ -82,7 +102,7 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 		case 'setExpirationDate':
 			if (isset($_POST['date'])) {
 				try {
-					$return = OCP\Share::setExpirationDate($_POST['itemType'], $_POST['itemSource'], $_POST['date']);
+					$return = OCP\Share::setExpirationDate((string)$_POST['itemType'], (string)$_POST['itemSource'], (string)$_POST['date']);
 					($return) ? OC_JSON::success() : OC_JSON::error();
 				} catch (\Exception $e) {
 					OC_JSON::error(array('data' => array('message' => $e->getMessage())));
@@ -90,11 +110,11 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 			}
 			break;
 		case 'informRecipients':
-			$l = OC_L10N::get('core');
+			$l = \OC::$server->getL10N('core');
 			$shareType = (int) $_POST['shareType'];
-			$itemType = $_POST['itemType'];
-			$itemSource = $_POST['itemSource'];
-			$recipient = $_POST['recipient'];
+			$itemType = (string)$_POST['itemType'];
+			$itemSource = (string)$_POST['itemSource'];
+			$recipient = (string)$_POST['recipient'];
 
 			if($shareType === \OCP\Share::SHARE_TYPE_USER) {
 				$recipientList[] = $recipient;
@@ -104,7 +124,14 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 			// don't send a mail to the user who shared the file
 			$recipientList = array_diff($recipientList, array(\OCP\User::getUser()));
 
-			$mailNotification = new OC\Share\MailNotifications(\OCP\User::getUser());
+			$mailNotification = new \OC\Share\MailNotifications(
+				\OC::$server->getUserSession()->getUser()->getUID(),
+				\OC::$server->getConfig(),
+				\OC::$server->getL10N('lib'),
+				\OC::$server->getMailer(),
+				\OC::$server->getLogger(),
+				$defaults
+			);
 			$result = $mailNotification->sendInternalShareMail($recipientList, $itemSource, $itemType);
 
 			\OCP\Share::setSendMailStatus($itemType, $itemSource, $shareType, $recipient, true);
@@ -122,26 +149,33 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 			}
 			break;
 		case 'informRecipientsDisabled':
-			$itemSource = $_POST['itemSource'];
-			$shareType = $_POST['shareType'];
-			$itemType = $_POST['itemType'];
-			$recipient = $_POST['recipient'];
+			$itemSource = (string)$_POST['itemSource'];
+			$shareType = (int)$_POST['shareType'];
+			$itemType = (string)$_POST['itemType'];
+			$recipient = (string)$_POST['recipient'];
 			\OCP\Share::setSendMailStatus($itemType, $itemSource, $shareType, $recipient, false);
 			OCP\JSON::success();
 			break;
 
 		case 'email':
 			// read post variables
-			$link = $_POST['link'];
-			$file = $_POST['file'];
-			$to_address = $_POST['toaddress'];
+			$link = (string)$_POST['link'];
+			$file = (string)$_POST['file'];
+			$to_address = (string)$_POST['toaddress'];
 
-			$mailNotification = new \OC\Share\MailNotifications(\OCP\User::getUser());
+			$mailNotification = new \OC\Share\MailNotifications(
+				\OC::$server->getUserSession()->getUser()->getUID(),
+				\OC::$server->getConfig(),
+				\OC::$server->getL10N('lib'),
+				\OC::$server->getMailer(),
+				\OC::$server->getLogger(),
+				$defaults
+			);
 
 			$expiration = null;
 			if (isset($_POST['expiration']) && $_POST['expiration'] !== '') {
 				try {
-					$date = new DateTime($_POST['expiration']);
+					$date = new DateTime((string)$_POST['expiration']);
 					$expiration = $date->getTimestamp();
 				} catch (Exception $e) {
 					\OCP\Util::writeLog('sharing', "Couldn't read date: " . $e->getMessage(), \OCP\Util::ERROR);
@@ -153,7 +187,7 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 			if(empty($result)) {
 				\OCP\JSON::success();
 			} else {
-				$l = OC_L10N::get('core');
+				$l = \OC::$server->getL10N('core');
 				OCP\JSON::error(array(
 					'data' => array(
 						'message' => $l->t("Couldn't send mail to following users: %s ",
@@ -169,7 +203,7 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 	switch ($_GET['fetch']) {
 		case 'getItemsSharedStatuses':
 			if (isset($_GET['itemType'])) {
-				$return = OCP\Share::getItemsShared($_GET['itemType'], OCP\Share::FORMAT_STATUSES);
+				$return = OCP\Share::getItemsShared((string)$_GET['itemType'], OCP\Share::FORMAT_STATUSES);
 				is_array($return) ? OC_JSON::success(array('data' => $return)) : OC_JSON::error();
 			}
 			break;
@@ -180,8 +214,8 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 				&& isset($_GET['checkShares'])) {
 				if ($_GET['checkReshare'] == 'true') {
 					$reshare = OCP\Share::getItemSharedWithBySource(
-						$_GET['itemType'],
-						$_GET['itemSource'],
+						(string)$_GET['itemType'],
+						(string)$_GET['itemSource'],
 						OCP\Share::FORMAT_NONE,
 						null,
 						true
@@ -191,8 +225,8 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 				}
 				if ($_GET['checkShares'] == 'true') {
 					$shares = OCP\Share::getItemShared(
-						$_GET['itemType'],
-						$_GET['itemSource'],
+						(string)$_GET['itemType'],
+						(string)$_GET['itemSource'],
 						OCP\Share::FORMAT_NONE,
 						null,
 						true
@@ -208,7 +242,7 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 			if (isset($_GET['search'])) {
 				$cm = OC::$server->getContactsManager();
 				if (!is_null($cm) && $cm->isEnabled()) {
-					$contacts = $cm->search($_GET['search'], array('FN', 'EMAIL'));
+					$contacts = $cm->search((string)$_GET['search'], array('FN', 'EMAIL'));
 					foreach ($contacts as $contact) {
 						if (!isset($contact['EMAIL'])) {
 							continue;
@@ -235,41 +269,46 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 			if (isset($_GET['search'])) {
 				$shareWithinGroupOnly = OC\Share\Share::shareWithGroupMembersOnly();
 				$shareWith = array();
-// 				if (OC_App::isEnabled('contacts')) {
-// 					// TODO Add function to contacts to only get the 'fullname' column to improve performance
-// 					$ids = OC_Contacts_Addressbook::activeIds();
-// 					foreach ($ids as $id) {
-// 						$vcards = OC_Contacts_VCard::all($id);
-// 						foreach ($vcards as $vcard) {
-// 							$contact = $vcard['fullname'];
-// 							if (stripos($contact, $_GET['search']) !== false
-// 								&& (!isset($_GET['itemShares'])
-// 								|| !isset($_GET['itemShares'][OCP\Share::SHARE_TYPE_CONTACT])
-// 								|| !is_array($_GET['itemShares'][OCP\Share::SHARE_TYPE_CONTACT])
-// 								|| !in_array($contact, $_GET['itemShares'][OCP\Share::SHARE_TYPE_CONTACT]))) {
-// 								$shareWith[] = array('label' => $contact, 'value' => array('shareType' => 5, 'shareWith' => $vcard['id']));
-// 							}
-// 						}
-// 					}
-// 				}
-				$groups = OC_Group::getGroups($_GET['search']);
+				$groups = OC_Group::getGroups((string)$_GET['search']);
 				if ($shareWithinGroupOnly) {
 					$usergroups = OC_Group::getUserGroups(OC_User::getUser());
 					$groups = array_intersect($groups, $usergroups);
 				}
+
+				$sharedUsers = [];
+				$sharedGroups = [];
+				if (isset($_GET['itemShares'])) {
+					if (isset($_GET['itemShares'][OCP\Share::SHARE_TYPE_USER]) &&
+					    is_array($_GET['itemShares'][OCP\Share::SHARE_TYPE_USER])) {
+						$sharedUsers = $_GET['itemShares'][OCP\Share::SHARE_TYPE_USER];
+					}
+
+					if (isset($_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP]) &&
+					    is_array($_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP])) {
+						$sharedGroups = $_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP];
+					}
+				}
+
 				$count = 0;
 				$users = array();
 				$limit = 0;
 				$offset = 0;
-				while ($count < 15 && count($users) == $limit) {
-					$limit = 15 - $count;
+				// limit defaults to 15 if not specified via request parameter and can be no larger than 500
+				$request_limit = min((int)$_GET['limit'] ?: 15, 500);
+				while ($count < $request_limit && count($users) == $limit) {
+					$limit = $request_limit - $count;
 					if ($shareWithinGroupOnly) {
-						$users = OC_Group::DisplayNamesInGroups($usergroups, $_GET['search'], $limit, $offset);
+						$users = OC_Group::displayNamesInGroups($usergroups, (string)$_GET['search'], $limit, $offset);
 					} else {
-						$users = OC_User::getDisplayNames($_GET['search'], $limit, $offset);
+						$users = OC_User::getDisplayNames((string)$_GET['search'], $limit, $offset);
 					}
+
 					$offset += $limit;
 					foreach ($users as $uid => $displayName) {
+						if (in_array($uid, $sharedUsers)) {
+							continue;
+						}
+
 						if ((!isset($_GET['itemShares'])
 							|| !is_array($_GET['itemShares'][OCP\Share::SHARE_TYPE_USER])
 							|| !in_array($uid, $_GET['itemShares'][OCP\Share::SHARE_TYPE_USER]))
@@ -287,10 +326,14 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 				$count = 0;
 
 				// enable l10n support
-				$l = OC_L10N::get('core');
+				$l = \OC::$server->getL10N('core');
 
 				foreach ($groups as $group) {
-					if ($count < 15) {
+					if (in_array($group, $sharedGroups)) {
+						continue;
+					}
+
+					if ($count < $request_limit) {
 						if (!isset($_GET['itemShares'])
 							|| !isset($_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP])
 							|| !is_array($_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP])
@@ -308,9 +351,50 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 						break;
 					}
 				}
-				$sorter = new \OC\Share\SearchResultSorter($_GET['search'],
+
+				// allow user to add unknown remote addresses for server-to-server share
+				$backend = \OCP\Share::getBackend((string)$_GET['itemType']);
+				if ($backend->isShareTypeAllowed(\OCP\Share::SHARE_TYPE_REMOTE)) {
+					if (substr_count((string)$_GET['search'], '@') >= 1) {
+						$shareWith[] = array(
+							'label' => (string)$_GET['search'],
+							'value' => array(
+								'shareType' => \OCP\Share::SHARE_TYPE_REMOTE,
+								'shareWith' => (string)$_GET['search']
+							)
+						);
+					}
+					$contactManager = \OC::$server->getContactsManager();
+					$addressBookContacts = $contactManager->search($_GET['search'], ['CLOUD', 'FN']);
+					foreach ($addressBookContacts as $contact) {
+						if (isset($contact['CLOUD'])) {
+							foreach ($contact['CLOUD'] as $cloudId) {
+								$shareWith[] = array(
+									'label' => $contact['FN'] . ' (' . $cloudId . ')',
+									'value' => array(
+										'shareType' => \OCP\Share::SHARE_TYPE_REMOTE,
+										'shareWith' => $cloudId
+									)
+								);
+							}
+						}
+					}
+				}
+
+				$sharingAutocompletion = \OC::$server->getConfig()
+					->getAppValue('core', 'shareapi_allow_share_dialog_user_enumeration', 'yes');
+
+				if ($sharingAutocompletion !== 'yes') {
+					$searchTerm = strtolower($_GET['search']);
+					$shareWith = array_filter($shareWith, function($user) use ($searchTerm) {
+						return strtolower($user['label']) === $searchTerm
+							|| strtolower($user['value']['shareWith']) === $searchTerm;
+					});
+				}
+
+				$sorter = new \OC\Share\SearchResultSorter((string)$_GET['search'],
 														   'label',
-														   new \OC\Log());
+														   \OC::$server->getLogger());
 				usort($shareWith, array($sorter, 'sort'));
 				OC_JSON::success(array('data' => $shareWith));
 			}
