@@ -134,6 +134,47 @@ describe('Core base tests', function() {
 			expect(escapeHTML('This is a good string without HTML.')).toEqual('This is a good string without HTML.');
 		});
 	});
+	describe('joinPaths', function() {
+		it('returns empty string with no or empty arguments', function() {
+			expect(OC.joinPaths()).toEqual('');
+			expect(OC.joinPaths('')).toEqual('');
+			expect(OC.joinPaths('', '')).toEqual('');
+		});
+		it('returns joined path sections', function() {
+			expect(OC.joinPaths('abc')).toEqual('abc');
+			expect(OC.joinPaths('abc', 'def')).toEqual('abc/def');
+			expect(OC.joinPaths('abc', 'def', 'ghi')).toEqual('abc/def/ghi');
+		});
+		it('keeps leading slashes', function() {
+			expect(OC.joinPaths('/abc')).toEqual('/abc');
+			expect(OC.joinPaths('/abc', 'def')).toEqual('/abc/def');
+			expect(OC.joinPaths('/abc', 'def', 'ghi')).toEqual('/abc/def/ghi');
+		});
+		it('keeps trailing slashes', function() {
+			expect(OC.joinPaths('abc/')).toEqual('abc/');
+			expect(OC.joinPaths('abc', 'def/')).toEqual('abc/def/');
+			expect(OC.joinPaths('abc', 'def', 'ghi/')).toEqual('abc/def/ghi/');
+		});
+		it('splits paths in specified strings and discards extra slashes', function() {
+			expect(OC.joinPaths('//abc//')).toEqual('/abc/');
+			expect(OC.joinPaths('//abc//def//')).toEqual('/abc/def/');
+			expect(OC.joinPaths('//abc//', '//def//')).toEqual('/abc/def/');
+			expect(OC.joinPaths('//abc//', '//def//', '//ghi//')).toEqual('/abc/def/ghi/');
+			expect(OC.joinPaths('//abc//def//', '//ghi//jkl/mno/', '//pqr//'))
+				.toEqual('/abc/def/ghi/jkl/mno/pqr/');
+			expect(OC.joinPaths('/abc', '/def')).toEqual('/abc/def');
+			expect(OC.joinPaths('/abc/', '/def')).toEqual('/abc/def');
+			expect(OC.joinPaths('/abc/', 'def')).toEqual('/abc/def');
+		});
+		it('discards empty sections', function() {
+			expect(OC.joinPaths('abc', '', 'def')).toEqual('abc/def');
+		});
+		it('returns root if only slashes', function() {
+			expect(OC.joinPaths('//')).toEqual('/');
+			expect(OC.joinPaths('/', '/')).toEqual('/');
+			expect(OC.joinPaths('/', '//', '/')).toEqual('/');
+		});
+	});
 	describe('filePath', function() {
 		beforeEach(function() {
 			OC.webroot = 'http://localhost';
@@ -393,8 +434,20 @@ describe('Core base tests', function() {
 			expect(OC.generateUrl('heartbeat')).toEqual(OC.webroot + '/index.php/heartbeat');
 			expect(OC.generateUrl('/heartbeat')).toEqual(OC.webroot + '/index.php/heartbeat');
 		});
-		it('substitutes parameters', function() {
-			expect(OC.generateUrl('apps/files/download{file}', {file: '/Welcome.txt'})).toEqual(OC.webroot + '/index.php/apps/files/download/Welcome.txt');
+		it('substitutes parameters which are escaped by default', function() {
+			expect(OC.generateUrl('apps/files/download/{file}', {file: '<">ImAnUnescapedString/!'})).toEqual(OC.webroot + '/index.php/apps/files/download/%3C%22%3EImAnUnescapedString%2F!');
+		});
+		it('substitutes parameters which can also be unescaped via option flag', function() {
+			expect(OC.generateUrl('apps/files/download/{file}', {file: 'subfolder/Welcome.txt'}, {escape: false})).toEqual(OC.webroot + '/index.php/apps/files/download/subfolder/Welcome.txt');
+		});
+		it('substitutes multiple parameters which are escaped by default', function() {
+			expect(OC.generateUrl('apps/files/download/{file}/{id}', {file: '<">ImAnUnescapedString/!', id: 5})).toEqual(OC.webroot + '/index.php/apps/files/download/%3C%22%3EImAnUnescapedString%2F!/5');
+		});
+		it('substitutes multiple parameters which can also be unescaped via option flag', function() {
+			expect(OC.generateUrl('apps/files/download/{file}/{id}', {file: 'subfolder/Welcome.txt', id: 5}, {escape: false})).toEqual(OC.webroot + '/index.php/apps/files/download/subfolder/Welcome.txt/5');
+		});
+		it('doesnt error out with no params provided', function  () {
+			expect(OC.generateUrl('apps/files/download{file}')).toEqual(OC.webroot + '/index.php/apps/files/download%7Bfile%7D');
 		});
 	});
 	describe('Main menu mobile toggle', function() {
@@ -406,7 +459,10 @@ describe('Core base tests', function() {
 		beforeEach(function() {
 			clock = sinon.useFakeTimers();
 			$('#testArea').append('<div id="header">' +
-				'<a class="menutoggle" href="#"></a>' +
+				'<a class="menutoggle" href="#">' +
+				'<h1 class="header-appname"></h1>' +
+				'<div class="icon-caret"></div>' +
+				'</a>' +
 				'</div>' +
 				'<div id="navigation"></div>');
 			$toggle = $('#header').find('.menutoggle');
@@ -495,5 +551,292 @@ describe('Core base tests', function() {
 			});
 		});
 	});
+	describe('naturalSortCompare', function() {
+		// cit() will skip tests if running on PhantomJS because it has issues with
+		// localeCompare(). See https://github.com/ariya/phantomjs/issues/11063
+		//
+		// Please make sure to run these tests in Chrome/Firefox manually
+		// to make sure they run.
+		var cit = window.isPhantom?xit:it;
+
+		// must provide the same results as \OC_Util::naturalSortCompare
+		it('sorts alphabetically', function() {
+			var a = [
+				'def',
+				'xyz',
+				'abc'
+			];
+			a.sort(OC.Util.naturalSortCompare);
+			expect(a).toEqual([
+				'abc',
+				'def',
+				'xyz'
+			]);
+		});
+		cit('sorts with different casing', function() {
+			var a = [
+				'aaa',
+				'bbb',
+				'BBB',
+				'AAA'
+			];
+			a.sort(OC.Util.naturalSortCompare);
+			expect(a).toEqual([
+				'aaa',
+				'AAA',
+				'bbb',
+				'BBB'
+			]);
+		});
+		it('sorts with numbers', function() {
+			var a = [
+				'124.txt',
+				'abc1',
+				'123.txt',
+				'abc',
+				'abc2',
+				'def (2).txt',
+				'ghi 10.txt',
+				'abc12',
+				'def.txt',
+				'def (1).txt',
+				'ghi 2.txt',
+				'def (10).txt',
+				'abc10',
+				'def (12).txt',
+				'z',
+				'ghi.txt',
+				'za',
+				'ghi 1.txt',
+				'ghi 12.txt',
+				'zz',
+				'15.txt',
+				'15b.txt'
+			];
+			a.sort(OC.Util.naturalSortCompare);
+			expect(a).toEqual([
+				'15.txt',
+				'15b.txt',
+				'123.txt',
+				'124.txt',
+				'abc',
+				'abc1',
+				'abc2',
+				'abc10',
+				'abc12',
+				'def.txt',
+				'def (1).txt',
+				'def (2).txt',
+				'def (10).txt',
+				'def (12).txt',
+				'ghi.txt',
+				'ghi 1.txt',
+				'ghi 2.txt',
+				'ghi 10.txt',
+				'ghi 12.txt',
+				'z',
+				'za',
+				'zz'
+			]);
+		});
+		it('sorts with chinese characters', function() {
+			var a = [
+				'十.txt',
+				'一.txt',
+				'二.txt',
+				'十 2.txt',
+				'三.txt',
+				'四.txt',
+				'abc.txt',
+				'五.txt',
+				'七.txt',
+				'八.txt',
+				'九.txt',
+				'六.txt',
+				'十一.txt',
+				'波.txt',
+				'破.txt',
+				'莫.txt',
+				'啊.txt',
+				'123.txt'
+			];
+			a.sort(OC.Util.naturalSortCompare);
+			expect(a).toEqual([
+				'123.txt',
+				'abc.txt',
+				'一.txt',
+				'七.txt',
+				'三.txt',
+				'九.txt',
+				'二.txt',
+				'五.txt',
+				'八.txt',
+				'六.txt',
+				'十.txt',
+				'十 2.txt',
+				'十一.txt',
+				'啊.txt',
+				'四.txt',
+				'波.txt',
+				'破.txt',
+				'莫.txt'
+			]);
+		});
+		cit('sorts with umlauts', function() {
+			var a = [
+				'öh.txt',
+				'Äh.txt',
+				'oh.txt',
+				'Üh 2.txt',
+				'Üh.txt',
+				'ah.txt',
+				'Öh.txt',
+				'uh.txt',
+				'üh.txt',
+				'äh.txt'
+			];
+			a.sort(OC.Util.naturalSortCompare);
+			expect(a).toEqual([
+				'ah.txt',
+				'äh.txt',
+				'Äh.txt',
+				'oh.txt',
+				'öh.txt',
+				'Öh.txt',
+				'uh.txt',
+				'üh.txt',
+				'Üh.txt',
+				'Üh 2.txt'
+			]);
+		});
+	});
+	describe('Plugins', function() {
+		var plugin;
+
+		beforeEach(function() {
+			plugin = {
+				name: 'Some name',
+				attach: function(obj) {
+					obj.attached = true;
+				},
+
+				detach: function(obj) {
+					obj.attached = false;
+				}
+			};
+			OC.Plugins.register('OC.Test.SomeName', plugin);
+		});
+		it('attach plugin to object', function() {
+			var obj = {something: true};
+			OC.Plugins.attach('OC.Test.SomeName', obj);
+			expect(obj.attached).toEqual(true);
+			OC.Plugins.detach('OC.Test.SomeName', obj);
+			expect(obj.attached).toEqual(false);
+		});
+		it('only call handler for target name', function() {
+			var obj = {something: true};
+			OC.Plugins.attach('OC.Test.SomeOtherName', obj);
+			expect(obj.attached).not.toBeDefined();
+			OC.Plugins.detach('OC.Test.SomeOtherName', obj);
+			expect(obj.attached).not.toBeDefined();
+		});
+	});
+	describe('Notifications', function() {
+		beforeEach(function(){
+			notificationMock = sinon.mock(OC.Notification);
+		});
+		afterEach(function(){
+			// verify that all expectations are met
+			notificationMock.verify();
+			// restore mocked methods
+			notificationMock.restore();
+			// clean up the global variable
+			delete notificationMock;
+		});
+		it('Should show a plain text notification' , function() {
+			// one is shown ...
+			notificationMock.expects('show').once().withExactArgs('My notification test');
+			// ... but not the HTML one
+			notificationMock.expects('showHtml').never();
+
+			OC.Notification.showTemporary('My notification test');
+
+			// verification is done in afterEach
+		});
+		it('Should show a HTML notification' , function() {
+			// no plain is shown ...
+			notificationMock.expects('show').never();
+			// ... but one HTML notification
+			notificationMock.expects('showHtml').once().withExactArgs('<a>My notification test</a>');
+
+			OC.Notification.showTemporary('<a>My notification test</a>', { isHTML: true });
+
+			// verification is done in afterEach
+		});
+		it('Should hide previous notification and hide itself after 7 seconds' , function() {
+			var clock = sinon.useFakeTimers();
+
+			// previous notifications get hidden
+			notificationMock.expects('hide').once();
+
+			OC.Notification.showTemporary('');
+
+			// verify the first call
+			notificationMock.verify();
+
+			// expect it a second time
+			notificationMock.expects('hide').once();
+
+			// travel in time +7000 milliseconds
+			clock.tick(7000);
+
+			// verification is done in afterEach
+		});
+		it('Should hide itself after a given time' , function() {
+			var clock = sinon.useFakeTimers();
+
+			// previous notifications get hidden
+			notificationMock.expects('hide').once();
+
+			OC.Notification.showTemporary('', { timeout: 10 });
+
+			// verify the first call
+			notificationMock.verify();
+
+			// expect to not be called after 9 seconds
+			notificationMock.expects('hide').never();
+
+			// travel in time +9 seconds
+			clock.tick(9000);
+			// verify this
+			notificationMock.verify();
+
+			// expect the second call one second later
+			notificationMock.expects('hide').once();
+			// travel in time +1 seconds
+			clock.tick(1000);
+
+			// verification is done in afterEach
+		});
+		it('Should not hide itself after a given time if a timeout of 0 is defined' , function() {
+			var clock = sinon.useFakeTimers();
+
+			// previous notifications get hidden
+			notificationMock.expects('hide').once();
+
+			OC.Notification.showTemporary('', { timeout: 0 });
+
+			// verify the first call
+			notificationMock.verify();
+
+			// expect to not be called after 1000 seconds
+			notificationMock.expects('hide').never();
+
+			// travel in time +1000 seconds
+			clock.tick(1000000);
+
+			// verification is done in afterEach
+		});
+	});
 });
 
