@@ -1,22 +1,29 @@
 <?php
 /**
- * ownCloud
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Joas Schilling <nickvergessen@owncloud.com>
+ * @author Miguel Prokop <miguel.prokop@vtu.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Vincent Petry <pvince81@owncloud.com>
  *
- * @author Bjoern Schiessle
- * @copyright 2014 Bjoern Schiessle <schiessle@owncloud.com>
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or any later version.
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
- * You should have received a copy of the GNU Affero General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 namespace OC\Share;
@@ -46,34 +53,19 @@ class Helper extends \OC\Share\Constants {
 			}
 			return $backend->generateTarget($itemSource, false);
 		} else {
-			if ($itemType == 'file' || $itemType == 'folder') {
-				$column = 'file_target';
-				$columnSource = 'file_source';
-			} else {
-				$column = 'item_target';
-				$columnSource = 'item_source';
-			}
 			if ($shareType == self::SHARE_TYPE_USER) {
 				// Share with is a user, so set share type to user and groups
 				$shareType = self::$shareTypeUserAndGroups;
 			}
-			$exclude = array();
-
-			$result = \OCP\Share::getItemsSharedWithUser($itemType, $shareWith);
-			foreach ($result as $row) {
-				if ($row['permissions'] > 0) {
-					$exclude[] = $row[$column];
-				}
-			}
 
 			// Check if suggested target exists first
 			if (!isset($suggestedTarget)) {
 				$suggestedTarget = $itemSource;
 			}
 			if ($shareType == self::SHARE_TYPE_GROUP) {
-				$target = $backend->generateTarget($suggestedTarget, false, $exclude);
+				$target = $backend->generateTarget($suggestedTarget, false);
 			} else {
-				$target = $backend->generateTarget($suggestedTarget, $shareWith, $exclude);
+				$target = $backend->generateTarget($suggestedTarget, $shareWith);
 			}
 
 			return $target;
@@ -166,14 +158,16 @@ class Helper extends \OC\Share\Constants {
 	 */
 	public static function getDefaultExpireSetting() {
 
+		$config = \OC::$server->getConfig();
+
 		$defaultExpireSettings = array('defaultExpireDateSet' => false);
 
 		// get default expire settings
-		$defaultExpireDate = \OC_Appconfig::getValue('core', 'shareapi_default_expire_date', 'no');
+		$defaultExpireDate = $config->getAppValue('core', 'shareapi_default_expire_date', 'no');
 		if ($defaultExpireDate === 'yes') {
-			$enforceExpireDate = \OC_Appconfig::getValue('core', 'shareapi_enforce_expire_date', 'no');
+			$enforceExpireDate = $config->getAppValue('core', 'shareapi_enforce_expire_date', 'no');
 			$defaultExpireSettings['defaultExpireDateSet'] = true;
-			$defaultExpireSettings['expireAfterDays'] = (int)\OC_Appconfig::getValue('core', 'shareapi_expire_after_n_days', '7');
+			$defaultExpireSettings['expireAfterDays'] = (int)($config->getAppValue('core', 'shareapi_expire_after_n_days', '7'));
 			$defaultExpireSettings['enforceExpireDate'] = $enforceExpireDate === 'yes' ? true : false;
 		}
 
@@ -225,56 +219,25 @@ class Helper extends \OC\Share\Constants {
 	}
 
 	/**
-	 * Extracts the necessary remote name from a given link
+	 * Strips away a potential file names and trailing slashes:
+	 * - http://localhost
+	 * - http://localhost/
+	 * - http://localhost/index.php
+	 * - http://localhost/index.php/s/{shareToken}
 	 *
-	 * Strips away a potential file name, to allow
-	 * - user
-	 * - user@localhost
-	 * - user@http://localhost
-	 * - user@http://localhost/
-	 * - user@http://localhost/index.php
-	 * - user@http://localhost/index.php/s/{shareToken}
+	 * all return: http://localhost
 	 *
 	 * @param string $shareWith
 	 * @return string
 	 */
-	public static function fixRemoteURLInShareWith($shareWith) {
-		if (strpos($shareWith, '@')) {
-			list($user, $remote) = explode('@', $shareWith, 2);
-
-			$remote = str_replace('\\', '/', $remote);
-			if ($fileNamePosition = strpos($remote, '/index.php')) {
-				$remote = substr($remote, 0, $fileNamePosition);
-			}
-			$remote = rtrim($remote, '/');
-
-			$shareWith = $user . '@' . $remote;
+	protected static function fixRemoteURL($remote) {
+		$remote = str_replace('\\', '/', $remote);
+		if ($fileNamePosition = strpos($remote, '/index.php')) {
+			$remote = substr($remote, 0, $fileNamePosition);
 		}
+		$remote = rtrim($remote, '/');
 
-		return rtrim($shareWith, '/');
-	}
-
-	/**
-	 * check if two federated cloud IDs refer to the same user
-	 *
-	 * @param string $user1
-	 * @param string $server1
-	 * @param string $user2
-	 * @param string $server2
-	 * @return bool true if both users and servers are the same
-	 */
-	public static function isSameUserOnSameServer($user1, $server1, $user2, $server2) {
-		$normalizedServer1 = strtolower(\OC\Share\Share::removeProtocolFromUrl($server1));
-		$normalizedServer2 = strtolower(\OC\Share\Share::removeProtocolFromUrl($server2));
-
-		if (
-				rtrim($normalizedServer1, '/') === rtrim($normalizedServer2, '/') &&
-				strtolower($user1) === strtolower($user2)
-		){
-			return true;
-		}
-
-		return false;
+		return $remote;
 	}
 
 	/**
@@ -328,24 +291,25 @@ class Helper extends \OC\Share\Constants {
 	}
 
 	/**
-	 * Strips away a potential file names and trailing slashes:
-	 * - http://localhost
-	 * - http://localhost/
-	 * - http://localhost/index.php
-	 * - http://localhost/index.php/s/{shareToken}
+	 * check if two federated cloud IDs refer to the same user
 	 *
-	 * all return: http://localhost
-	 *
-	 * @param string $remote
-	 * @return string
+	 * @param string $user1
+	 * @param string $server1
+	 * @param string $user2
+	 * @param string $server2
+	 * @return bool true if both users and servers are the same
 	 */
-	protected static function fixRemoteURL($remote) {
-		$remote = str_replace('\\', '/', $remote);
-		if ($fileNamePosition = strpos($remote, '/index.php')) {
-			$remote = substr($remote, 0, $fileNamePosition);
-		}
-		$remote = rtrim($remote, '/');
+	public static function isSameUserOnSameServer($user1, $server1, $user2, $server2) {
+		$normalizedServer1 = strtolower(\OC\Share\Share::removeProtocolFromUrl($server1));
+		$normalizedServer2 = strtolower(\OC\Share\Share::removeProtocolFromUrl($server2));
 
-		return $remote;
+		if (
+				rtrim($normalizedServer1, '/') === rtrim($normalizedServer2, '/') &&
+				strtolower($user1) === strtolower($user2)
+		) {
+				return true;
+		}
+
+		return false;
 	}
 }
