@@ -1,9 +1,33 @@
 <?php
 /**
- * Copyright (c) 2013 Robin Appelman <icewind@owncloud.com>
- * This file is licensed under the Affero General Public License version 3 or
- * later.
- * See the COPYING-README file.
+ * @author Arthur Schiwon <blizzz@owncloud.com>
+ * @author Bart Visscher <bartv@thisnet.nl>
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Frank Karlitschek <frank@owncloud.org>
+ * @author Lukas Reschke <lukas@owncloud.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
+ * @author Steffen Lindner <mail@steffen-lindner.de>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Victor Dubiniuk <dubiniuk@owncloud.com>
+ * @author Vincent Petry <pvince81@owncloud.com>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
  */
 
 namespace OC;
@@ -11,8 +35,11 @@ namespace OC;
 use OC\Core\Command\Log\Manage;
 use OC\Hooks\BasicEmitter;
 use OC_App;
+use OC_Installer;
 use OC_Util;
 use OCP\IConfig;
+use OC\Setup;
+use OCP\ILogger;
 
 /**
  * Class that handles autoupdating of ownCloud
@@ -25,7 +52,7 @@ use OCP\IConfig;
  */
 class Updater extends BasicEmitter {
 
-	/** @var \OC\Log $log */
+	/** @var ILogger $log */
 	private $log;
 	
 	/** @var \OC\HTTPHelper $helper */
@@ -54,9 +81,11 @@ class Updater extends BasicEmitter {
 	/**
 	 * @param HTTPHelper $httpHelper
 	 * @param IConfig $config
-	 * @param \OC\Log $log
+	 * @param ILogger $log
 	 */
-	public function __construct(HTTPHelper $httpHelper, IConfig $config, $log = null) {
+	public function __construct(HTTPHelper $httpHelper,
+								IConfig $config,
+								ILogger $log = null) {
 		$this->httpHelper = $httpHelper;
 		$this->log = $log;
 		$this->config = $config;
@@ -98,23 +127,20 @@ class Updater extends BasicEmitter {
 	/**
 	 * Check if a new version is available
 	 *
-	 * @param string $updaterUrl the url to check, i.e. 'https://updates.owncloud.com/server/'
 	 * @return array|bool
 	 */
-	public function check($updaterUrl = null) {
+	public function check() {
 
 		// Look up the cache - it is invalidated all 30 minutes
 		if (((int)$this->config->getAppValue('core', 'lastupdatedat') + 1800) > time()) {
 			return json_decode($this->config->getAppValue('core', 'lastupdateResult'), true);
 		}
 
-		if (is_null($updaterUrl)) {
-			$updaterUrl = 'https://updates.owncloud.com/server/';
-		}
+		$updaterUrl = $this->config->getSystemValue('updater.server.url', 'https://updates.owncloud.com/server/');
 
 		$this->config->setAppValue('core', 'lastupdatedat', time());
 
-		if ($this->config->getAppValue('core', 'installedat', '') == '') {
+		if ($this->config->getAppValue('core', 'installedat', '') === '') {
 			$this->config->setAppValue('core', 'installedat', microtime(true));
 		}
 
@@ -129,9 +155,7 @@ class Updater extends BasicEmitter {
 		//fetch xml data from updater
 		$url = $updaterUrl . '?version=' . $versionString;
 
-		// set a sensible timeout of 10 sec to stay responsive even if the update server is down.
-
-		$tmp = array();
+		$tmp = [];
 		$xml = $this->httpHelper->getUrlContent($url);
 		if ($xml) {
 			$loadEntities = libxml_disable_entity_loader(true);
@@ -144,7 +168,7 @@ class Updater extends BasicEmitter {
 				$tmp['web'] = (string)$data->web;
 			}
 		} else {
-			$data = array();
+			$data = [];
 		}
 
 		// Cache the result
@@ -163,14 +187,18 @@ class Updater extends BasicEmitter {
 		$this->emit('\OC\Updater', 'setDebugLogLevel', [ $logLevel, $this->logLevelNames[$logLevel] ]);
 		$this->config->setSystemValue('loglevel', \OCP\Util::DEBUG);
 
-		$this->config->setSystemValue('maintenance', true);
+		$wasMaintenanceModeEnabled = $this->config->getSystemValue('maintenance', false);
+
+		if(!$wasMaintenanceModeEnabled) {
+			$this->config->setSystemValue('maintenance', true);
+			$this->emit('\OC\Updater', 'maintenanceEnabled');
+		}
 
 		$installedVersion = $this->config->getSystemValue('version', '0.0.0');
 		$currentVersion = implode('.', \OC_Util::getVersion());
 		if ($this->log) {
 			$this->log->debug('starting upgrade from ' . $installedVersion . ' to ' . $currentVersion, array('app' => 'core'));
 		}
-		$this->emit('\OC\Updater', 'maintenanceStart');
 
 		$success = true;
 		try {
@@ -181,30 +209,42 @@ class Updater extends BasicEmitter {
 			$success = false;
 		}
 
-		$this->config->setSystemValue('maintenance', false);
-		$this->emit('\OC\Updater', 'maintenanceEnd');
 		$this->emit('\OC\Updater', 'updateEnd', array($success));
 
+		if(!$wasMaintenanceModeEnabled && $success) {
+			$this->config->setSystemValue('maintenance', false);
+			$this->emit('\OC\Updater', 'maintenanceDisabled');
+		} else {
+			$this->emit('\OC\Updater', 'maintenanceActive');
+		}
+
 		$this->emit('\OC\Updater', 'resetLogLevel', [ $logLevel, $this->logLevelNames[$logLevel] ]);
 		$this->config->setSystemValue('loglevel', $logLevel);
 
 		return $success;
 	}
 
+	/**
+	 * Return version from which this version is allowed to upgrade from
+	 *
+	 * @return string allowed previous version
+	 */
+	private function getAllowedPreviousVersion() {
+		// this should really be a JSON file
+		require \OC::$SERVERROOT . '/version.php';
+		return implode('.', $OC_VersionCanBeUpgradedFrom);
+	}
+
 	/**
 	 * Whether an upgrade to a specified version is possible
 	 * @param string $oldVersion
 	 * @param string $newVersion
+	 * @param string $allowedPreviousVersion
 	 * @return bool
 	 */
-	public function isUpgradePossible($oldVersion, $newVersion) {
-		$oldVersion = explode('.', $oldVersion);
-		$newVersion = explode('.', $newVersion);
-
-		if($newVersion[0] > ($oldVersion[0] + 1) || $oldVersion[0] > $newVersion[0]) {
-			return false;
-		}
-		return true;
+	public function isUpgradePossible($oldVersion, $newVersion, $allowedPreviousVersion) {
+		return (version_compare($allowedPreviousVersion, $oldVersion, '<=')
+			&& version_compare($oldVersion, $newVersion, '<='));
 	}
 
 	/**
@@ -219,6 +259,12 @@ class Updater extends BasicEmitter {
 		$repair->listen('\OC\Repair', 'error', function ($description) {
 			$this->emit('\OC\Updater', 'repairError', array($description));
 		});
+		$repair->listen('\OC\Repair', 'info', function ($description) {
+			$this->emit('\OC\Updater', 'repairInfo', array($description));
+		});
+		$repair->listen('\OC\Repair', 'step', function ($description) {
+			$this->emit('\OC\Updater', 'repairStep', array($description));
+		});
 	}
 
 	/**
@@ -233,17 +279,24 @@ class Updater extends BasicEmitter {
 	 */
 	private function doUpgrade($currentVersion, $installedVersion) {
 		// Stop update if the update is over several major versions
-		if (!self::isUpgradePossible($installedVersion, $currentVersion)) {
-			throw new \Exception('Updates between multiple major versions are unsupported.');
+		$allowedPreviousVersion = $this->getAllowedPreviousVersion();
+		if (!self::isUpgradePossible($installedVersion, $currentVersion, $allowedPreviousVersion)) {
+			throw new \Exception('Updates between multiple major versions and downgrades are unsupported.');
 		}
 
-		// Update htaccess files for apache hosts
-		if (isset($_SERVER['SERVER_SOFTWARE']) && strstr($_SERVER['SERVER_SOFTWARE'], 'Apache')) {
-			try {
-				\OC_Setup::updateHtaccess();
-			} catch (\Exception $e) {
-				throw new \Exception($e->getMessage());
-			}
+		// Update .htaccess files
+		try {
+			Setup::updateHtaccess();
+			Setup::protectDataDirectory();
+		} catch (\Exception $e) {
+			throw new \Exception($e->getMessage());
+		}
+
+		// FIXME: Some users do not upload the new ca-bundle.crt, let's catch this
+		// in the update. For a newer release we shall use an integrity check after
+		// the update.
+		if(!file_exists(\OC::$configDir .'/ca-bundle.crt')) {
+			throw new \Exception('Please upload the ca-bundle.crt file into the \'config\' directory.');
 		}
 
 		// create empty file in data dir, so we can later find
@@ -268,9 +321,14 @@ class Updater extends BasicEmitter {
 		if ($this->updateStepEnabled) {
 			$this->doCoreUpgrade();
 
-			$this->checkAppsRequirements();
+			// update all shipped apps
+			$disabledApps = $this->checkAppsRequirements();
 			$this->doAppUpgrade();
 
+			// upgrade appstore apps
+			$this->upgradeAppStoreApps($disabledApps);
+
+
 			// post-upgrade repairs
 			$repair = new Repair(Repair::getRepairSteps());
 			$this->emitRepairMessages($repair);
@@ -336,7 +394,6 @@ class Updater extends BasicEmitter {
 		include \OC_App::getAppPath($appId) . '/appinfo/preupdate.php';
 	}
 
-
 	/**
 	 * upgrades all apps within a major ownCloud upgrade. Also loads "priority"
 	 * (types authentication, filesystem, logging, in that order) afterwards.
@@ -368,6 +425,7 @@ class Updater extends BasicEmitter {
 		foreach ($stacks as $type => $stack) {
 			foreach ($stack as $appId) {
 				if (\OC_App::shouldUpgrade($appId)) {
+					$this->emit('\OC\Updater', 'appUpgradeStarted', array($appId, \OC_App::getAppVersion($appId)));
 					\OC_App::updateApp($appId);
 					$this->emit('\OC\Updater', 'appUpgrade', array($appId, \OC_App::getAppVersion($appId)));
 				}
@@ -386,11 +444,15 @@ class Updater extends BasicEmitter {
 	 * ownCloud version. disable them if not.
 	 * This is important if you upgrade ownCloud and have non ported 3rd
 	 * party apps installed.
+	 *
+	 * @return array
+	 * @throws \Exception
 	 */
 	private function checkAppsRequirements() {
 		$isCoreUpgrade = $this->isCodeUpgrade();
 		$apps = OC_App::getEnabledApps();
 		$version = OC_Util::getVersion();
+		$disabledApps = [];
 		foreach ($apps as $app) {
 			// check if the app is compatible with this version of ownCloud
 			$info = OC_App::getAppInfo($app);
@@ -414,11 +476,16 @@ class Updater extends BasicEmitter {
 			// disable any other 3rd party apps if not overriden
 			if(!$this->skip3rdPartyAppsDisable) {
 				\OC_App::disable($app);
+				$disabledApps[]= $app;
 				$this->emit('\OC\Updater', 'thirdPartyAppDisabled', array($app));
 			};
 		}
+		return $disabledApps;
 	}
 
+	/**
+	 * @return bool
+	 */
 	private function isCodeUpgrade() {
 		$installedVersion = $this->config->getSystemValue('version', '0.0.0');
 		$currentVersion = implode('.', OC_Util::getVersion());
@@ -427,5 +494,20 @@ class Updater extends BasicEmitter {
 		}
 		return false;
 	}
+
+	/**
+	 * @param array $disabledApps
+	 * @throws \Exception
+	 */
+	private function upgradeAppStoreApps(array $disabledApps) {
+		foreach($disabledApps as $app) {
+			if (OC_Installer::isUpdateAvailable($app)) {
+				$ocsId = \OC::$server->getConfig()->getAppValue($app, 'ocsid', '');
+
+				$this->emit('\OC\Updater', 'upgradeAppStoreApp', array($app));
+				OC_Installer::updateAppByOCSId($ocsId);
+			}
+		}
+	}
 }
 
