@@ -1,22 +1,41 @@
 <?php
 /**
- * ownCloud
+ * @author Aldo "xoen" Giambelluca <xoen@xoen.org>
+ * @author Andreas Fischer <bantu@owncloud.com>
+ * @author Arthur Schiwon <blizzz@owncloud.com>
+ * @author Bartek Przybylski <bart.p.pl@gmail.com>
+ * @author Bart Visscher <bartv@thisnet.nl>
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Dominik Schmidt <dev@dominik-schmidt.de>
+ * @author Florian Preinstorfer <nblock@archlinux.us>
+ * @author Georg Ehrke <georg@owncloud.com>
+ * @author Jakob Sack <mail@jakobsack.de>
+ * @author Joas Schilling <nickvergessen@owncloud.com>
+ * @author Jörn Friedrich Dreyer <jfd@butonic.de>
+ * @author Lukas Reschke <lukas@owncloud.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
+ * @author Scrutinizer Auto-Fixer <auto-fixer@scrutinizer-ci.com>
+ * @author shkdee <louis.traynard@m4x.org>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Tom Needham <tom@owncloud.com>
+ * @author Victor Dubiniuk <dubiniuk@owncloud.com>
  *
- * @author Frank Karlitschek
- * @copyright 2012 Frank Karlitschek frank@owncloud.org
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or any later version.
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU Affero General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -47,6 +66,7 @@ class OC_User {
 
 	/**
 	 * @return \OC\User\Manager
+	 * @deprecated Use \OC::$server->getUserManager()
 	 */
 	public static function getManager() {
 		return OC::$server->getUserManager();
@@ -63,8 +83,9 @@ class OC_User {
 
 	/**
 	 * registers backend
+	 *
 	 * @param string $backend name of the backend
-	 * @deprecated Add classes by calling useBackend with a class instance instead
+	 * @deprecated Add classes by calling OC_User::useBackend() with a class instance instead
 	 * @return bool
 	 *
 	 * Makes a list of backends that can be used by other modules
@@ -76,6 +97,7 @@ class OC_User {
 
 	/**
 	 * gets available backends
+	 *
 	 * @deprecated
 	 * @return array an array of backends
 	 *
@@ -87,6 +109,7 @@ class OC_User {
 
 	/**
 	 * gets used backends
+	 *
 	 * @deprecated
 	 * @return array an array of backends
 	 *
@@ -98,6 +121,7 @@ class OC_User {
 
 	/**
 	 * Adds the backend to the list of used backends
+	 *
 	 * @param string|OC_User_Interface $backend default: database The backend to use for user management
 	 * @return bool
 	 *
@@ -170,6 +194,7 @@ class OC_User {
 
 	/**
 	 * Create a new user
+	 *
 	 * @param string $uid The username of the user to create
 	 * @param string $password The password of the new user
 	 * @throws Exception
@@ -179,6 +204,7 @@ class OC_User {
 	 * itself, not in its subclasses.
 	 *
 	 * Allowed characters in the username are: "a-z", "A-Z", "0-9" and "_.@-"
+	 * @deprecated Use \OC::$server->getUserManager()->createUser($uid, $password)
 	 */
 	public static function createUser($uid, $password) {
 		return self::getManager()->createUser($uid, $password);
@@ -186,37 +212,17 @@ class OC_User {
 
 	/**
 	 * delete a user
+	 *
 	 * @param string $uid The username of the user to delete
 	 * @return bool
 	 *
 	 * Deletes a user
+	 * @deprecated Use \OC::$server->getUserManager()->get() and then run delete() on the return
 	 */
 	public static function deleteUser($uid) {
 		$user = self::getManager()->get($uid);
 		if ($user) {
-			$result = $user->delete();
-
-			// if delete was successful we clean-up the rest
-			if ($result) {
-
-				// We have to delete the user from all groups
-				foreach (OC_Group::getUserGroups($uid) as $i) {
-					OC_Group::removeFromGroup($uid, $i);
-				}
-				// Delete the user's keys in preferences
-				OC_Preferences::deleteUser($uid);
-
-				// Delete user files in /data/
-				OC_Helper::rmdirr(\OC_User::getHome($uid));
-
-				// Delete the users entry in the storage table
-				\OC\Files\Cache\Storage::remove('home::' . $uid);
-
-				// Remove it from the Cache
-				self::getManager()->delete($uid);
-			}
-
-			return $result;
+			return $user->delete();
 		} else {
 			return false;
 		}
@@ -224,6 +230,7 @@ class OC_User {
 
 	/**
 	 * Try to login a user
+	 *
 	 * @param string $loginname The login name of the user to log in
 	 * @param string $password The password of the user
 	 * @return boolean|null
@@ -231,14 +238,14 @@ class OC_User {
 	 * Log in a user and regenerate a new session - if the password is ok
 	 */
 	public static function login($loginname, $password) {
-		$loginname = str_replace("\0", '', $loginname);
-		$password = str_replace("\0", '', $password);
-
 		session_regenerate_id(true);
 		$result = self::getUserSession()->login($loginname, $password);
 		if ($result) {
 			//we need to pass the user name, which may differ from login name
-			OC_Util::setupFS(self::getUserSession()->getUser()->getUID());
+			$user = self::getUserSession()->getUser()->getUID();
+			OC_Util::setupFS($user);
+			//trigger creation of user home and /files folder
+			\OC::$server->getUserFolder($user);
 		}
 		return $result;
 	}
@@ -267,14 +274,16 @@ class OC_User {
 
 		$uid = $backend->getCurrentUserId();
 		$run = true;
-		OC_Hook::emit( "OC_User", "pre_login", array( "run" => &$run, "uid" => $uid ));
+		OC_Hook::emit("OC_User", "pre_login", array("run" => &$run, "uid" => $uid));
 
-		if($uid) {
-			self::setUserId($uid);
-			self::setDisplayName($uid);
-			self::getUserSession()->setLoginName($uid);
+		if ($uid) {
+			if (self::getUser() !== $uid) {
+				self::setUserId($uid);
+				self::setDisplayName($uid);
+				self::getUserSession()->setLoginName($uid);
 
-			OC_Hook::emit( "OC_User", "post_login", array( "uid" => $uid, 'password'=>'' ));
+				OC_Hook::emit("OC_User", "post_login", array("uid" => $uid, 'password' => ''));
+			}
 			return true;
 		}
 		return false;
@@ -308,11 +317,18 @@ class OC_User {
 	 * Sets user id for session and triggers emit
 	 */
 	public static function setUserId($uid) {
-		OC::$session->set('user_id', $uid);
+		$userSession = \OC::$server->getUserSession();
+		$userManager = \OC::$server->getUserManager();
+		if ($user = $userManager->get($uid)) {
+			$userSession->setUser($user);
+		} else {
+			\OC::$server->getSession()->set('user_id', $uid);
+		}
 	}
 
 	/**
 	 * Sets user display name for session
+	 *
 	 * @param string $uid
 	 * @param null $displayName
 	 * @return bool Whether the display name could get set
@@ -338,18 +354,23 @@ class OC_User {
 		self::getUserSession()->logout();
 	}
 
+	/**
+	 * Tries to login the user with HTTP Basic Authentication
+	 */
+	public static function tryBasicAuthLogin() {
+		if (!empty($_SERVER['PHP_AUTH_USER']) && !empty($_SERVER['PHP_AUTH_PW'])) {
+			\OC_User::login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']);
+		}
+	}
+
 	/**
 	 * Check if the user is logged in, considers also the HTTP basic credentials
+	 *
 	 * @return bool
 	 */
 	public static function isLoggedIn() {
-		if (\OC::$session->get('user_id') !== null && self::$incognitoMode === false) {
-			return self::userExists(\OC::$session->get('user_id'));
-		}
-
-		// Check whether the user has authenticated using Basic Authentication
-		if (isset($_SERVER['PHP_AUTH_USER']) && isset($_SERVER['PHP_AUTH_PW'])) {
-			return \OC_User::login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']);
+		if (\OC::$server->getSession()->get('user_id') !== null && self::$incognitoMode === false) {
+			return self::userExists(\OC::$server->getSession()->get('user_id'));
 		}
 
 		return false;
@@ -357,11 +378,20 @@ class OC_User {
 
 	/**
 	 * set incognito mode, e.g. if a user wants to open a public link
+	 *
 	 * @param bool $status
 	 */
 	public static function setIncognitoMode($status) {
 		self::$incognitoMode = $status;
+	}
 
+	/**
+	 * get incognito mode status
+	 *
+	 * @return bool
+	 */
+	public static function isIncognitoMode() {
+		return self::$incognitoMode;
 	}
 
 	/**
@@ -377,11 +407,12 @@ class OC_User {
 			return $backend->getLogoutAttribute();
 		}
 
-		return 'href="' . link_to('', 'index.php') . '?logout=true&requesttoken=' . OC_Util::callRegister() . '"';
+		return 'href="' . link_to('', 'index.php') . '?logout=true&requesttoken=' . urlencode(OC_Util::callRegister()) . '"';
 	}
 
 	/**
 	 * Check if the user is an admin user
+	 *
 	 * @param string $uid uid of the admin
 	 * @return bool
 	 */
@@ -395,10 +426,11 @@ class OC_User {
 
 	/**
 	 * get the user id of the user currently logged in.
-	 * @return string uid or false
+	 *
+	 * @return string|bool uid or false
 	 */
 	public static function getUser() {
-		$uid = OC::$session ? OC::$session->get('user_id') : null;
+		$uid = \OC::$server->getSession() ? \OC::$server->getSession()->get('user_id') : null;
 		if (!is_null($uid) && self::$incognitoMode === false) {
 			return $uid;
 		} else {
@@ -408,6 +440,7 @@ class OC_User {
 
 	/**
 	 * get the display name of the user currently logged in.
+	 *
 	 * @param string $uid
 	 * @return string uid or false
 	 */
@@ -431,16 +464,18 @@ class OC_User {
 
 	/**
 	 * Autogenerate a password
+	 *
 	 * @return string
 	 *
 	 * generates a password
 	 */
 	public static function generatePassword() {
-		return OC_Util::generateRandomBytes(30);
+		return \OC::$server->getSecureRandom()->getMediumStrengthGenerator()->generate(30);
 	}
 
 	/**
 	 * Set password
+	 *
 	 * @param string $uid The username
 	 * @param string $password The new password
 	 * @param string $recoveryPassword for the encryption app to reset encryption keys
@@ -459,6 +494,7 @@ class OC_User {
 
 	/**
 	 * Check whether user can change his avatar
+	 *
 	 * @param string $uid The username
 	 * @return bool
 	 *
@@ -475,6 +511,7 @@ class OC_User {
 
 	/**
 	 * Check whether user can change his password
+	 *
 	 * @param string $uid The username
 	 * @return bool
 	 *
@@ -491,6 +528,7 @@ class OC_User {
 
 	/**
 	 * Check whether user can change his display name
+	 *
 	 * @param string $uid The username
 	 * @return bool
 	 *
@@ -507,6 +545,7 @@ class OC_User {
 
 	/**
 	 * Check if the password is correct
+	 *
 	 * @param string $uid The username
 	 * @param string $password The password
 	 * @return string|false user id a string on success, false otherwise
@@ -528,6 +567,7 @@ class OC_User {
 	 * @return string
 	 *
 	 * returns the path to the users home directory
+	 * @deprecated Use \OC::$server->getUserManager->getHome()
 	 */
 	public static function getHome($uid) {
 		$user = self::getManager()->get($uid);
@@ -540,6 +580,7 @@ class OC_User {
 
 	/**
 	 * Get a list of all users
+	 *
 	 * @return array an array of all uids
 	 *
 	 * Get a list of all users.
@@ -558,12 +599,14 @@ class OC_User {
 
 	/**
 	 * Get a list of all users display name
+	 *
 	 * @param string $search
 	 * @param int $limit
 	 * @param int $offset
 	 * @return array associative array with all display names (value) and corresponding uids (key)
 	 *
 	 * Get a list of all display names and user ids.
+	 * @deprecated Use \OC::$server->getUserManager->searchDisplayName($search, $limit, $offset) instead.
 	 */
 	public static function getDisplayNames($search = '', $limit = null, $offset = null) {
 		$displayNames = array();
@@ -576,6 +619,7 @@ class OC_User {
 
 	/**
 	 * check if a user exists
+	 *
 	 * @param string $uid the username
 	 * @return boolean
 	 */
@@ -624,6 +668,7 @@ class OC_User {
 
 	/**
 	 * Set cookie value to use in next page load
+	 *
 	 * @param string $username username to be set
 	 * @param string $token
 	 */
@@ -640,6 +685,7 @@ class OC_User {
 
 	/**
 	 * Returns the first active backend from self::$_usedBackends.
+	 *
 	 * @return OCP\Authentication\IApacheBackend|null if no backend active, otherwise OCP\Authentication\IApacheBackend
 	 */
 	private static function findFirstActiveUsedBackend() {
