@@ -7,6 +7,7 @@
 /**
  * @var array $_
  * @var \OCP\IL10N $l
+ * @var OC_Defaults $theme
  */
 
 style('settings', 'settings');
@@ -15,32 +16,32 @@ script('core', ['multiselect', 'setupchecks']);
 vendor_script('select2/select2');
 vendor_style('select2/select2');
 
-$levels = array('Debug', 'Info', 'Warning', 'Error', 'Fatal');
-$levelLabels = array(
+$levels = ['Debug', 'Info', 'Warning', 'Error', 'Fatal'];
+$levelLabels = [
 	$l->t( 'Everything (fatal issues, errors, warnings, info, debug)' ),
 	$l->t( 'Info, warnings, errors and fatal issues' ),
 	$l->t( 'Warnings, errors and fatal issues' ),
 	$l->t( 'Errors and fatal issues' ),
 	$l->t( 'Fatal issues only' ),
-);
+];
 
-$mail_smtpauthtype = array(
+$mail_smtpauthtype = [
 	''	=> $l->t('None'),
 	'LOGIN'	=> $l->t('Login'),
 	'PLAIN'	=> $l->t('Plain'),
 	'NTLM'	=> $l->t('NT LAN Manager'),
-);
+];
 
-$mail_smtpsecure = array(
+$mail_smtpsecure = [
 	''		=> $l->t('None'),
 	'ssl'	=> $l->t('SSL'),
 	'tls'	=> $l->t('TLS'),
-);
+];
 
-$mail_smtpmode = array(
+$mail_smtpmode = [
 	'php',
 	'smtp',
-);
+];
 if ($_['sendmail_is_available']) {
 	$mail_smtpmode[] = 'sendmail';
 }
@@ -63,139 +64,79 @@ if ($_['mail_smtpmode'] == 'qmail') {
 
 <div id="app-content">
 
-<div id="security-warning">
+<div id="security-warning" class="section">
+	<h2><?php p($l->t('Security & setup warnings'));?></h2>
+	<ul>
 <?php
-
-// is ssl working ?
-if (!$_['isConnectedViaHTTPS']) {
-	?>
-<div class="section">
-	<h2><?php p($l->t('Security Warning'));?></h2>
-
-	<span class="securitywarning">
-		<?php p($l->t('You are accessing %s via HTTP. We strongly suggest you configure your server to require using HTTPS instead.', $theme->getTitle())); ?>
-	</span>
-
-</div>
+// is php setup properly to query system environment variables like getenv('PATH')
+if ($_['getenvServerNotWorking']) {
+?>
+	<li>
+		<?php p($l->t('php does not seem to be setup properly to query system environment variables. The test with getenv("PATH") only returns an empty response.')); ?><br>
+		<?php p($l->t('Please check the installation documentation for php configuration notes and the php configuration of your server, especially when using php-fpm.')); ?>
+	</li>
 <?php
 }
 
 // is read only config enabled
 if ($_['readOnlyConfigEnabled']) {
 ?>
-<div class="section">
-	<h2><?php p($l->t('Read-Only config enabled'));?></h2>
-
-	<span class="securitywarning">
+	<li>
 		<?php p($l->t('The Read-Only config has been enabled. This prevents setting some configurations via the web-interface. Furthermore, the file needs to be made writable manually for every update.')); ?>
-	</span>
-
-	</div>
+	</li>
 <?php
 }
+
 // Are doc blocks accessible?
 if (!$_['isAnnotationsWorking']) {
 	?>
-<div class="section">
-	<h2><?php p($l->t('Setup Warning'));?></h2>
-
-	<span class="securitywarning">
-		<?php p($l->t('PHP is apparently setup to strip inline doc blocks. This will make several core apps inaccessible.')); ?>
+	<li>
+		<?php p($l->t('PHP is apparently setup to strip inline doc blocks. This will make several core apps inaccessible.')); ?><br>
 		<?php p($l->t('This is probably caused by a cache/accelerator such as Zend OPcache or eAccelerator.')); ?>
-	</span>
-
-</div>
+	</li>
 <?php
 }
 
-// SQLite database performance issue
-if ($_['databaseOverload']) {
+// Is the Transaction isolation level READ_COMMITED?
+if ($_['invalidTransactionIsolationLevel']) {
 	?>
-<div class="section">
-	<h2><?php p($l->t('Database Performance Info'));?></h2>
-
-	<p>
-		<strong>
-			<?php p($l->t('SQLite is used as database. For larger installations we recommend to switch to a different database backend.')); ?>
-		</strong>
-	</p>
-	<p>
-		<strong>
-			<?php p($l->t('Especially when using the desktop client for file syncing the use of SQLite is discouraged.')); ?>
-		</strong>
-	</p>
-	<p>
-		<?php p($l->t('To migrate to another database use the command line tool: \'occ db:convert-type\'')); ?>
-	</p>
-
-</div>
+	<li>
+		<?php p($l->t('Your database does not run with "READ COMMITED" transaction isolation level. This can cause problems when multiple actions are executed in parallel.')); ?>
+	</li>
 <?php
 }
 
 // Windows Warning
 if ($_['WindowsWarning']) {
 	?>
-<div class="section">
-	<h2><?php p($l->t('Microsoft Windows Platform'));?></h2>
-
-	<p class="securitywarning">
+	<li>
 		<?php p($l->t('Your server is running on Microsoft Windows. We highly recommend Linux for optimal user experience.')); ?>
-	</p>
-
-</div>
-
+	</li>
 <?php
 }
 
-// APCU Warning if outdated
-if ($_['ApcuOutdatedWarning']) {
+// Warning if memcache is outdated
+foreach ($_['OutdatedCacheWarning'] as $php_module => $data) {
 	?>
-	<div class="section">
-		<h2><?php p($l->t('APCu below version 4.0.6 installed'));?></h2>
-
-		<p class="securitywarning">
-			<?php p($l->t('APCu below version 4.0.6 is installed, for stability and performance reasons we recommend to update to a newer APCu version.')); ?>
-		</p>
-
-	</div>
-
+	<li>
+		<?php p($l->t('%1$s below version %2$s is installed, for stability and performance reasons we recommend to update to a newer %1$s version.', $data)); ?>
+	</li>
 <?php
 }
+
 // if module fileinfo available?
 if (!$_['has_fileinfo']) {
 	?>
-<div class="section">
-	<h2><?php p($l->t('Module \'fileinfo\' missing'));?></h2>
-
-		<span class="connectionwarning">
+	<li>
 		<?php p($l->t('The PHP module \'fileinfo\' is missing. We strongly recommend to enable this module to get best results with mime-type detection.')); ?>
-	</span>
-
-</div>
-<?php
-}
-
-// is PHP charset set to UTF8?
-if (!$_['isPhpCharSetUtf8']) {
-	?>
-	<div class="section">
-		<h2><?php p($l->t('PHP charset is not set to UTF-8'));?></h2>
-
-		<span class="connectionwarning">
-		<?php p($l->t("PHP charset is not set to UTF-8. This can cause major issues with non-ASCII characters in file names. We highly recommend to change the value of 'default_charset' php.ini to 'UTF-8'.")); ?>
-	</span>
-
-	</div>
+	</li>
 <?php
 }
 
 // is locale working ?
 if (!$_['isLocaleWorking']) {
 	?>
-<div class="section">
-	<h2><?php p($l->t('Locale not working'));?></h2>
-
-		<span class="connectionwarning">
+	<li>
 		<?php
 			$locales = 'en_US.UTF-8/fr_FR.UTF-8/es_ES.UTF-8/de_DE.UTF-8/ru_RU.UTF-8/pt_BR.UTF-8/it_IT.UTF-8/ja_JP.UTF-8/zh_CN.UTF-8';
 			p($l->t('System locale can not be set to a one which supports UTF-8.'));
@@ -206,61 +147,124 @@ if (!$_['isLocaleWorking']) {
 		?>
 			<br>
 			<?php
-			p($l->t('We strongly suggest installing the required packages on your system to support one of the following locales: %s.', array($locales)));
+			p($l->t('We strongly suggest installing the required packages on your system to support one of the following locales: %s.', [$locales]));
 			?>
-	</span>
-
-</div>
+	</li>
 <?php
 }
 
 if ($_['suggestedOverwriteCliUrl']) {
 	?>
-	<div class="section">
-		<h2><?php p($l->t('URL generation in notification emails'));?></h2>
-
-		<span class="connectionwarning">
+	<li>
 		<?php p($l->t('If your installation is not installed in the root of the domain and uses system cron, there can be issues with the URL generation. To avoid these problems, please set the "overwrite.cli.url" option in your config.php file to the webroot path of your installation (Suggested: "%s")', $_['suggestedOverwriteCliUrl'])); ?>
-	</span>
-
-	</div>
+	</li>
 <?php
 }
 
 if ($_['cronErrors']) {
 	?>
-	<div class="section">
-		<h2><?php p($l->t('Cronjob encountered misconfiguration'));?></h2>
-
-		<span class="connectionwarning">
+	<li>
 			<?php p($l->t('It was not possible to execute the cronjob via CLI. The following technical errors have appeared:')); ?>
-			<br/>
+			<br>
 			<ol>
 				<?php foreach(json_decode($_['cronErrors']) as $error) { if(isset($error->error)) {?>
-					<li><?php p($error->error) ?></li>
-					<ul><li><?php p($error->hint) ?></li></ul>
-
+					<li><?php p($error->error) ?> <?php p($error->hint) ?></li>
 				<?php }};?>
 			</ol>
-		</span>
-
-	</div>
+	</li>
 <?php
 }
 ?>
+</ul>
 
-<div id="postsetupchecks" class="section">
-	<h2><?php p($l->t('Configuration Checks'));?></h2>
+<div id="postsetupchecks">
 	<div class="loading"></div>
-	<div class="success hidden"><?php p($l->t('No problems found'));?></div>
-	<div class="errors hidden"></div>
-	<div class="hint hidden">
-		<span class="setupwarning"><?php
-			print_unescaped($l->t('Please double check the <a href=\'%s\'>installation guides</a>.', \OC_Helper::linkToDocs('admin-install')));
-		?></span>
+	<ul class="errors hidden"></ul>
+	<p class="hint hidden">
+		<?php print_unescaped($l->t('Please double check the <a target="_blank" href="%s">installation guides ↗</a>, and check for any errors or warnings in the <a href="#log-section">log</a>.', link_to_docs('admin-install'))); ?>
+	</p>
+</div>
+</div>
+
+	<div class="section" id="shareAPI">
+		<h2><?php p($l->t('Sharing'));?></h2>
+		<a target="_blank" class="icon-info svg"
+			title="<?php p($l->t('Open documentation'));?>"
+			href="<?php p(link_to_docs('admin-sharing')); ?>"></a>
+		<p id="enable">
+			<input type="checkbox" name="shareapi_enabled" id="shareAPIEnabled"
+				   value="1" <?php if ($_['shareAPIEnabled'] === 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="shareAPIEnabled"><?php p($l->t('Allow apps to use the Share API'));?></label><br/>
+		</p>
+		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
+			<input type="checkbox" name="shareapi_allow_links" id="allowLinks"
+				   value="1" <?php if ($_['allowLinks'] === 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="allowLinks"><?php p($l->t('Allow users to share via link'));?></label><br/>
+		</p>
+
+		<p id="publicLinkSettings" class="indent <?php if ($_['allowLinks'] !== 'yes' || $_['shareAPIEnabled'] === 'no') p('hidden'); ?>">
+			<input type="checkbox" name="shareapi_enforce_links_password" id="enforceLinkPassword"
+				   value="1" <?php if ($_['enforceLinkPassword']) print_unescaped('checked="checked"'); ?> />
+			<label for="enforceLinkPassword"><?php p($l->t('Enforce password protection'));?></label><br/>
+
+			<input type="checkbox" name="shareapi_allow_public_upload" id="allowPublicUpload"
+				   value="1" <?php if ($_['allowPublicUpload'] == 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="allowPublicUpload"><?php p($l->t('Allow public uploads'));?></label><br/>
+
+			<input type="checkbox" name="shareapi_allow_public_notification" id="allowPublicMailNotification"
+				   value="1" <?php if ($_['allowPublicMailNotification'] == 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="allowPublicMailNotification"><?php p($l->t('Allow users to send mail notification for shared files'));?></label><br/>
+
+			<input type="checkbox" name="shareapi_default_expire_date" id="shareapiDefaultExpireDate"
+				   value="1" <?php if ($_['shareDefaultExpireDateSet'] === 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="shareapiDefaultExpireDate"><?php p($l->t('Set default expiration date'));?></label><br/>
+
+		</p>
+		<p id="setDefaultExpireDate" class="double-indent <?php if ($_['allowLinks'] !== 'yes' || $_['shareDefaultExpireDateSet'] === 'no' || $_['shareAPIEnabled'] === 'no') p('hidden');?>">
+			<?php p($l->t( 'Expire after ' )); ?>
+			<input type="text" name='shareapi_expire_after_n_days' id="shareapiExpireAfterNDays" placeholder="<?php p('7')?>"
+				   value='<?php p($_['shareExpireAfterNDays']) ?>' />
+			<?php p($l->t( 'days' )); ?>
+			<input type="checkbox" name="shareapi_enforce_expire_date" id="shareapiEnforceExpireDate"
+				   value="1" <?php if ($_['shareEnforceExpireDate'] === 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="shareapiEnforceExpireDate"><?php p($l->t('Enforce expiration date'));?></label><br/>
+		</p>
+		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
+			<input type="checkbox" name="shareapi_allow_resharing" id="allowResharing"
+				   value="1" <?php if ($_['allowResharing'] === 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="allowResharing"><?php p($l->t('Allow resharing'));?></label><br/>
+		</p>
+		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
+			<input type="checkbox" name="shareapi_only_share_with_group_members" id="onlyShareWithGroupMembers"
+				   value="1" <?php if ($_['onlyShareWithGroupMembers']) print_unescaped('checked="checked"'); ?> />
+			<label for="onlyShareWithGroupMembers"><?php p($l->t('Restrict users to only share with users in their groups'));?></label><br/>
+		</p>
+		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
+			<input type="checkbox" name="shareapi_allow_mail_notification" id="allowMailNotification"
+				   value="1" <?php if ($_['allowMailNotification'] === 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="allowMailNotification"><?php p($l->t('Allow users to send mail notification for shared files to other users'));?></label><br/>
+		</p>
+		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
+			<input type="checkbox" name="shareapi_exclude_groups" id="shareapiExcludeGroups"
+				   value="1" <?php if ($_['shareExcludeGroups']) print_unescaped('checked="checked"'); ?> />
+			<label for="shareapiExcludeGroups"><?php p($l->t('Exclude groups from sharing'));?></label><br/>
+		</p>
+		<p id="selectExcludedGroups" class="indent <?php if (!$_['shareExcludeGroups'] || $_['shareAPIEnabled'] === 'no') p('hidden'); ?>">
+			<input name="shareapi_exclude_groups_list" type="hidden" id="excludedGroups" value="<?php p($_['shareExcludedGroupsList']) ?>" style="width: 400px"/>
+			<br />
+			<em><?php p($l->t('These groups will still be able to receive shares, but not to initiate them.')); ?></em>
+		</p>
+		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
+			<input type="checkbox" name="shareapi_allow_share_dialog_user_enumeration" value="1" id="shareapi_allow_share_dialog_user_enumeration"
+				<?php if ($_['allowShareDialogUserEnumeration'] === 'yes') print_unescaped('checked="checked"'); ?> />
+			<label for="shareapi_allow_share_dialog_user_enumeration"><?php p($l->t('Allow username autocompletion in share dialog. If this is disabled the full username needs to be entered.'));?></label><br />
+		</p>
+
+		<?php print_unescaped($_['fileSharingSettings']); ?>
 	</div>
-</div>
-</div>
+
+<?php print_unescaped($_['filesExternal']); ?>
+
 <?php foreach($_['forms'] as $form) {
 	if (isset($form['form'])) {?>
 		<div id="<?php isset($form['anchor']) ? p($form['anchor']) : p('');?>"><?php print_unescaped($form['form']);?></div>
@@ -272,20 +276,29 @@ if ($_['cronErrors']) {
 	<?php if ($_['cron_log']): ?>
 	<p class="cronlog inlineblock">
 		<?php if ($_['lastcron'] !== false):
-			$human_time = OC_Util::formatDate($_['lastcron']);
+			$relative_time = relative_modified_date($_['lastcron']);
+			$absolute_time = OC_Util::formatDate($_['lastcron']);
 			if (time() - $_['lastcron'] <= 3600): ?>
 				<span class="cronstatus success"></span>
-				<?php p($l->t("Last cron was executed at %s.", array($human_time)));
-			else: ?>
+				<span class="crondate" original-title="<?php p($absolute_time);?>">
+					<?php p($l->t("Last cron job execution: %s.", [$relative_time]));?>
+				</span>
+			<?php else: ?>
 				<span class="cronstatus error"></span>
-				<?php p($l->t("Last cron was executed at %s. This is more than an hour ago, something seems wrong.", array($human_time)));
-			endif;
+				<span class="crondate" original-title="<?php p($absolute_time);?>">
+					<?php p($l->t("Last cron job execution: %s. Something seems wrong.", [$relative_time]));?>
+				</span>
+			<?php endif;
 		else: ?>
 			<span class="cronstatus error"></span>
 			<?php p($l->t("Cron was not executed yet!"));
 		endif; ?>
 	</p>
 	<?php endif; ?>
+	<a target="_blank" class="icon-info svg"
+		title="<?php p($l->t('Open documentation'));?>"
+		href="<?php p(link_to_docs('admin-background-jobs')); ?>"></a>
+
 	<p>
 				<input type="radio" name="mode" value="ajax"
 					   id="backgroundjobs_ajax" <?php if ($_['backgroundjobs_mode'] === "ajax") {
@@ -312,122 +325,70 @@ if ($_['cronErrors']) {
 	</p>
 </div>
 
-<div class="section" id="shareAPI">
-	<h2><?php p($l->t('Sharing'));?></h2>
-		<p id="enable">
-			<input type="checkbox" name="shareapi_enabled" id="shareAPIEnabled"
-				   value="1" <?php if ($_['shareAPIEnabled'] === 'yes') print_unescaped('checked="checked"'); ?> />
-			<label for="shareAPIEnabled"><?php p($l->t('Allow apps to use the Share API'));?></label><br/>
-		</p>
-		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_allow_links" id="allowLinks"
-				   value="1" <?php if ($_['allowLinks'] === 'yes') print_unescaped('checked="checked"'); ?> />
-			<label for="allowLinks"><?php p($l->t('Allow users to share via link'));?></label><br/>
-		</p>
+<div class="section" id='encryptionAPI'>
+	<h2><?php p($l->t('Server-side encryption')); ?></h2>
+	<a target="_blank" class="icon-info svg"
+		title="<?php p($l->t('Open documentation'));?>"
+		href="<?php p(link_to_docs('admin-encryption')); ?>"></a>
 
-			<p id="publicLinkSettings" class="indent <?php if ($_['allowLinks'] !== 'yes' || $_['shareAPIEnabled'] === 'no') p('hidden'); ?>">
-				<input type="checkbox" name="shareapi_enforce_links_password" id="enforceLinkPassword"
-						   value="1" <?php if ($_['enforceLinkPassword']) print_unescaped('checked="checked"'); ?> />
-				<label for="enforceLinkPassword"><?php p($l->t('Enforce password protection'));?></label><br/>
-
-				<input type="checkbox" name="shareapi_allow_public_upload" id="allowPublicUpload"
-				       value="1" <?php if ($_['allowPublicUpload'] == 'yes') print_unescaped('checked="checked"'); ?> />
-				<label for="allowPublicUpload"><?php p($l->t('Allow public uploads'));?></label><br/>
-
-				<input type="checkbox" name="shareapi_allow_public_notification" id="allowPublicMailNotification"
-					value="1" <?php if ($_['allowPublicMailNotification'] == 'yes') print_unescaped('checked="checked"'); ?> />
-				<label for="allowPublicMailNotification"><?php p($l->t('Allow users to send mail notification for shared files'));?></label><br/>
-
-				<input type="checkbox" name="shareapi_default_expire_date" id="shareapiDefaultExpireDate"
-				       value="1" <?php if ($_['shareDefaultExpireDateSet'] === 'yes') print_unescaped('checked="checked"'); ?> />
-				<label for="shareapiDefaultExpireDate"><?php p($l->t('Set default expiration date'));?></label><br/>
-
-			</p>
-				<p id="setDefaultExpireDate" class="double-indent <?php if ($_['allowLinks'] !== 'yes' || $_['shareDefaultExpireDateSet'] === 'no' || $_['shareAPIEnabled'] === 'no') p('hidden');?>">
-					<?php p($l->t( 'Expire after ' )); ?>
-					<input type="text" name='shareapi_expire_after_n_days' id="shareapiExpireAfterNDays" placeholder="<?php p('7')?>"
-						   value='<?php p($_['shareExpireAfterNDays']) ?>' />
-					<?php p($l->t( 'days' )); ?>
-					<input type="checkbox" name="shareapi_enforce_expire_date" id="shareapiEnforceExpireDate"
-						   value="1" <?php if ($_['shareEnforceExpireDate'] === 'yes') print_unescaped('checked="checked"'); ?> />
-					<label for="shareapiEnforceExpireDate"><?php p($l->t('Enforce expiration date'));?></label><br/>
-				</p>
-		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_allow_resharing" id="allowResharing"
-				   value="1" <?php if ($_['allowResharing'] === 'yes') print_unescaped('checked="checked"'); ?> />
-			<label for="allowResharing"><?php p($l->t('Allow resharing'));?></label><br/>
-		</p>
-		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_only_share_with_group_members" id="onlyShareWithGroupMembers"
-				   value="1" <?php if ($_['onlyShareWithGroupMembers']) print_unescaped('checked="checked"'); ?> />
-			<label for="onlyShareWithGroupMembers"><?php p($l->t('Restrict users to only share with users in their groups'));?></label><br/>
-		</p>
-		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_allow_mail_notification" id="allowMailNotification"
-				   value="1" <?php if ($_['allowMailNotification'] === 'yes') print_unescaped('checked="checked"'); ?> />
-			<label for="allowMailNotification"><?php p($l->t('Allow users to send mail notification for shared files to other users'));?></label><br/>
-		</p>
-		<p class="<?php if ($_['shareAPIEnabled'] === 'no') p('hidden');?>">
-			<input type="checkbox" name="shareapi_exclude_groups" id="shareapiExcludeGroups"
-			       value="1" <?php if ($_['shareExcludeGroups']) print_unescaped('checked="checked"'); ?> />
-			<label for="shareapiExcludeGroups"><?php p($l->t('Exclude groups from sharing'));?></label><br/>
-		</p>
-			<p id="selectExcludedGroups" class="indent <?php if (!$_['shareExcludeGroups'] || $_['shareAPIEnabled'] === 'no') p('hidden'); ?>">
-				<input name="shareapi_exclude_groups_list" type="hidden" id="excludedGroups" value="<?php p($_['shareExcludedGroupsList']) ?>" style="width: 400px"/>
-				<br />
-				<em><?php p($l->t('These groups will still be able to receive shares, but not to initiate them.')); ?></em>
-			</p>
-</div>
-
-<div class="section" id="security">
-	<h2><?php p($l->t('Security'));?></h2>
-	<p>
-		<input type="checkbox" name="forcessl"  id="forcessl"
-			<?php if ($_['enforceHTTPSEnabled']) {
-				print_unescaped('checked="checked" ');
-				print_unescaped('value="true"');
-			}  else {
-				print_unescaped('value="false"');
-			}
-			?>
-			<?php if (!$_['isConnectedViaHTTPS']) p('disabled'); ?> />
-		<label for="forcessl"><?php p($l->t('Enforce HTTPS'));?></label><br/>
-		<em><?php p($l->t(
-			'Forces the clients to connect to %s via an encrypted connection.',
-			$theme->getName()
-		)); ?></em><br/>
-		<span id="forceSSLforSubdomainsSpan" <?php if(!$_['enforceHTTPSEnabled']) { print_unescaped('class="hidden"'); } ?>>
-			<input type="checkbox" name="forceSSLforSubdomains"  id="forceSSLforSubdomains"
-				<?php if ($_['forceSSLforSubdomainsEnabled']) {
-					print_unescaped('checked="checked" ');
-					print_unescaped('value="true"');
-				}  else {
-					print_unescaped('value="false"');
-				}
-				?>
-				<?php if (!$_['isConnectedViaHTTPS']) { p('disabled'); } ?> />
-			<label for="forceSSLforSubdomains"><?php p($l->t('Enforce HTTPS for subdomains'));?></label><br/>
-			<em><?php p($l->t(
-					'Forces the clients to connect to %s and subdomains via an encrypted connection.',
-					$theme->getName()
-				)); ?></em>
-		</span>
-		<?php if (!$_['isConnectedViaHTTPS']) {
-			print_unescaped("<br/><em>");
-			p($l->t(
-				'Please connect to your %s via HTTPS to enable or disable the SSL enforcement.',
-				$theme->getName()
-			));
-			p(' Please note: You need to configure SSL within your webserver configuration first.');
-			print_unescaped("</em>");
-		}
-		?>
+	<p id="enable">
+		<input type="checkbox"
+			   id="enableEncryption"
+			   value="1" <?php if ($_['encryptionEnabled']) print_unescaped('checked="checked" disabled="disabled"'); ?> />
+		<label
+			for="enableEncryption"><?php p($l->t('Enable server-side encryption')); ?> <span id="startmigration_msg" class="msg"></span> </label><br/>
 	</p>
+
+	<div id="EncryptionWarning" class="warning hidden">
+		<?php p($l->t('Encryption is a one way process. Once encryption is enabled, all files from that point forward will be encrypted on the server and it will not be possible to disable encryption at a later date. This is the final warning: Do you really want to enable encryption?')) ?>
+		<input type="button"
+			   id="reallyEnableEncryption"
+			   value="<?php p($l->t("Enable encryption")); ?>" />
+	</div>
+
+	<div id="EncryptionSettingsArea" class="<?php if (!$_['encryptionEnabled']) p('hidden'); ?>">
+		<div id='selectEncryptionModules' class="<?php if (!$_['encryptionReady']) p('hidden'); ?>">
+			<?php
+			if (empty($_['encryptionModules'])) {
+				p($l->t('No encryption module loaded, please enable an encryption module in the app menu.'));
+			} else { ?>
+				<h3><?php p($l->t('Select default encryption module:')) ?></h3>
+				<fieldset id='encryptionModules'>
+					<?php foreach ($_['encryptionModules'] as $id => $module): ?>
+						<input type="radio" id="<?php p($id) ?>"
+							   name="default_encryption_module"
+							   value="<?php p($id) ?>"
+							<?php if ($module['default']) {
+								p('checked');
+							} ?>>
+						<label
+							for="<?php p($id) ?>"><?php p($module['displayName']) ?></label>
+						<br/>
+
+						<?php if ($id === 'OC_DEFAULT_MODULE') print_unescaped($_['ocDefaultEncryptionModulePanel']); ?>
+					<?php endforeach; ?>
+				</fieldset>
+			<?php } ?>
+		</div>
+		<div id="migrationWarning" class="<?php if ($_['encryptionReady']) p('hidden'); ?>">
+			<?php
+			if ($_['encryptionReady'] === false && $_['externalBackendsEnabled'] === true) {
+				p($l->t('You need to migrate your encryption keys from the old encryption (ownCloud <= 8.0) to the new one. Please enable the "Default encryption module" and run \'occ encryption:migrate\''));
+			} elseif ($_['encryptionReady'] === false && $_['externalBackendsEnabled'] === false) {
+				p($l->t('You need to migrate your encryption keys from the old encryption (ownCloud <= 8.0) to the new one.')); ?>
+				<input type="submit" name="startmigration" id="startmigration"
+					   value="<?php p($l->t('Start migration')); ?>"/>
+			<?php } ?>
+		</div>
+	</div>
 </div>
 
-<div class="section">
-	<form id="mail_general_settings" class="mail_settings">
-		<h2><?php p($l->t('Email Server'));?></h2>
+<div class="section" id="mail_general_settings">
+	<form id="mail_general_settings_form" class="mail_settings">
+		<h2><?php p($l->t('Email server'));?></h2>
+		<a target="_blank" class="icon-info svg"
+			title="<?php p($l->t('Open documentation'));?>"
+			href="<?php p(link_to_docs('admin-email')); ?>"></a>
 
 		<p><?php p($l->t('This is used for sending out notifications.')); ?> <span id="mail_settings_msg" class="msg"></span></p>
 
@@ -498,7 +459,7 @@ if ($_['cronErrors']) {
 			<label for="mail_smtpname"><?php p($l->t( 'Credentials' )); ?></label>
 			<input type="text" name='mail_smtpname' id="mail_smtpname" placeholder="<?php p($l->t('SMTP Username'))?>"
 				   value='<?php p($_['mail_smtpname']) ?>' />
-			<input type="password" name='mail_smtppassword' id="mail_smtppassword"
+			<input type="password" name='mail_smtppassword' id="mail_smtppassword" autocomplete="off"
 				   placeholder="<?php p($l->t('SMTP Password'))?>" value='<?php p($_['mail_smtppassword']) ?>' />
 			<input id="mail_credentials_settings_submit" type="button" value="<?php p($l->t('Store credentials')) ?>">
 		</p>
@@ -531,7 +492,7 @@ if ($_['cronErrors']) {
 			<td>
 				<?php p($entry->app);?>
 			</td>
-			<td>
+			<td class="log-message">
 				<?php p($entry->message);?>
 			</td>
 			<td class="date">
@@ -554,22 +515,56 @@ if ($_['cronErrors']) {
 	<?php if ($_['logFileSize'] > (100 * 1024 * 1024)): ?>
 	<br>
 	<em>
-		<?php p($l->t('The logfile is bigger than 100MB. Downloading it may take some time!')); ?>
+		<?php p($l->t('The logfile is bigger than 100 MB. Downloading it may take some time!')); ?>
 	</em>
 	<?php endif; ?>
 	<?php endif; ?>
 </div>
 
+<div class="section" id="admin-tips">
+	<h2><?php p($l->t('Tips & tricks'));?></h2>
+	<ul>
+		<?php
+		// SQLite database performance issue
+		if ($_['databaseOverload']) {
+			?>
+			<li>
+				<?php p($l->t('SQLite is used as database. For larger installations we recommend to switch to a different database backend.')); ?><br>
+				<?php p($l->t('Especially when using the desktop client for file syncing the use of SQLite is discouraged.')); ?><br>
+				<?php print_unescaped($l->t('To migrate to another database use the command line tool: \'occ db:convert-type\', or see the <a target="_blank" href="%s">documentation ↗</a>.', link_to_docs('admin-db-conversion') )); ?>
+			</li>
+		<?php } ?>
+		<li><a target="_blank" href="<?php p(link_to_docs('admin-backup')); ?>"><?php p($l->t('How to do backups'));?> ↗</a></li>
+		<li><a target="_blank" href="<?php p(link_to_docs('admin-monitoring')); ?>"><?php p($l->t('Advanced monitoring'));?> ↗</a></li>
+		<li><a target="_blank" href="<?php p(link_to_docs('admin-performance')); ?>"><?php p($l->t('Performance tuning'));?> ↗</a></li>
+		<li><a target="_blank" href="<?php p(link_to_docs('admin-config')); ?>"><?php p($l->t('Improving the config.php'));?> ↗</a></li>
+		<li><a target="_blank" href="<?php p(link_to_docs('developer-theming')); ?>"><?php p($l->t('Theming'));?> ↗</a></li>
+		<li><a target="_blank" href="<?php p(link_to_docs('admin-security')); ?>"><?php p($l->t('Hardening and security guidance'));?> ↗</a></li>
+	</ul>
+</div>
+<div class="section" id="server-status">
+	<h2><?php p($l->t('Server Status'));?></h2>
+	<ul>
+		<li>
+			<?php if ($_['fileLockingEnabled']) {
+				p($l->t('Transactional File Locking is enabled.'));
+			} else {
+				p($l->t('Transactional File Locking is disabled.'));
+			} ?>
+		</li>
+	</ul>
+</div>
+
 <div class="section">
 	<h2><?php p($l->t('Version'));?></h2>
 	<strong><?php p($theme->getTitle()); ?></strong> <?php p(OC_Util::getHumanVersion()) ?>
-<?php if (OC_Util::getEditionString() === ''): ?>
-	<p>
-		<?php print_unescaped($l->t('Developed by the <a href="http://ownCloud.org/contact" target="_blank">ownCloud community</a>, the <a href="https://github.com/owncloud" target="_blank">source code</a> is licensed under the <a href="http://www.gnu.org/licenses/agpl-3.0.html" target="_blank"><abbr title="Affero General Public License">AGPL</abbr></a>.')); ?>
-	</p>
-<?php endif; ?>
+	<?php include('settings.development.notice.php'); ?>
 </div>
 
+<?php if (!empty($_['updaterAppPanel'])): ?>
+	<div id="updater"><?php print_unescaped($_['updaterAppPanel']); ?></div>
+<?php endif; ?>
+
 <div class="section credits-footer">
 	<p><?php print_unescaped($theme->getShortFooter()); ?></p>
 </div>
